#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from collections import defaultdict

import tornado
from preggy import expect

from tests.base import TestCase
from thumbor.context import ServerParameters
from thumbor.importer import Importer
from thumbor.storages.mixed_storage import Storage as MixedStorage
from thumbor.storages.no_storage import Storage as NoStorage


class Storage:
    def __init__(self, security_key):
        self.storage = defaultdict(dict)
        self.security_key = security_key

    async def put(self, path, contents):
        self.storage[path]["path"] = path
        self.storage[path]["contents"] = contents

    async def put_crypto(self, path):
        self.storage[path]["crypto"] = self.security_key

    async def put_detector_data(self, path, data):
        self.storage[path]["detector"] = data

    async def get_crypto(self, path):
        if path not in self.storage:
            raise RuntimeError(f"{path} was not found in storage")

        return self.storage[path]["crypto"]

    async def get_detector_data(self, path):
        if path not in self.storage or "detector" not in self.storage[path]:
            return None

        return self.storage[path]["detector"]

    async def get(self, path):
        if path not in self.storage:
            raise RuntimeError(f"{path} was not found in storage")

        return self.storage[path]["contents"]


class BasedMixedStorageTestCase(TestCase):
    def __init__(self, *args, **kw):
        self.context = None
        self.storage = None
        super().__init__(*args, **kw)

    def get_storages(self):
        return (
            Storage("security-key"),
            Storage("security-key"),
            Storage("detector"),
        )

    def get_context(self):
        context = super().get_context()
        self.storage = MixedStorage(None, *self.get_storages())
        return context

    def get_server(self):
        server = ServerParameters(
            8888, "localhost", "thumbor.conf", None, "info", None
        )
        server.security_key = "ACME-SEC"
        return server

    def get_importer(self):
        return Importer(self.config)


class MixedStorageTestCase(BasedMixedStorageTestCase):
    @tornado.testing.gen_test
    async def test_mixed_storage_put_includes_path(self):
        await self.storage.put("path1", "contents")
        await self.storage.put_crypto("path1")
        await self.storage.put_detector_data("path1", "detector")

        expect(self.storage.file_storage.storage["path1"]["path"]).to_equal(
            "path1"
        )
        expect(
            self.storage.file_storage.storage["path1"]["contents"]
        ).to_equal("contents")

        contents = await self.storage.get("path1")
        expect(contents).to_equal("contents")
        expect(self.storage.file_storage.storage["path1"]).not_to_include(
            "crypto"
        )
        expect(self.storage.crypto_storage.storage["path1"]).not_to_include(
            "contents"
        )
        expect(
            self.storage.crypto_storage.storage["path1"]["crypto"]
        ).to_equal("security-key")

        contents = await self.storage.get_crypto("path1")
        expect(contents).to_equal("security-key")

        contents = await self.storage.get_detector_data("path1")
        expect(contents).to_equal("detector")


class MixedStorageFromConfTestCase(BasedMixedStorageTestCase):
    def get_context(self):
        context = super().get_context()
        self.storage = MixedStorage(context)
        return context

    @tornado.testing.gen_test
    async def test_get_data(self):
        path = await self.storage.get("path")
        _, _, _ = self.get_storages()
        expect(self.storage.file_storage).to_be_instance_of(NoStorage)
        expect(path).to_be_null()

    @tornado.testing.gen_test
    async def test_get_detector_data(self):
        path = await self.storage.get_detector_data("path")
        expect(self.storage.detector_storage).to_be_instance_of(NoStorage)
        expect(path).to_be_null()

    @tornado.testing.gen_test
    async def test_get_crypto(self):
        path = await self.storage.get_crypto("path")
        expect(self.storage.crypto_storage).to_be_instance_of(NoStorage)
        expect(path).to_be_null()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from os.path import expanduser
from unittest import TestCase, mock

from preggy import expect

from thumbor.config import Config, format_value, generate_config


class ConfigTestCase(TestCase):
    @mock.patch("derpconf.config.generate_config")
    def test_can_generate_config(self, config_mock):
        generate_config()
        expect(config_mock.called).to_be_true()

    def test_can_format_value(self):
        expect(format_value("qwe")).to_equal("'qwe'")
        expect(format_value(["qwe", "rty"])).to_equal("[\n#    qwe#    rty#]")
        expect(format_value(230)).to_equal(230)


class ConfigValuesTestCase(TestCase):
    def get_default_storage(self):
        return "thumbor.storages.file_storage"

    def get_config(self):
        return (
            ("MAX_WIDTH", 0),
            ("MAX_HEIGHT", 0),
            ("ALLOWED_SOURCES", []),
            ("QUALITY", 80),
            ("LOADER", "thumbor.loaders.http_loader"),
            ("STORAGE", self.get_default_storage()),
            ("ENGINE", "thumbor.engines.pil"),
            ("GIF_ENGINE", "thumbor.engines.gif"),
            ("URL_SIGNER", "libthumbor.url_signers.base64_hmac_sha1"),
            ("ALLOW_UNSAFE_URL", True),
            ("FILE_LOADER_ROOT_PATH", expanduser("~")),
            ("STORAGE_EXPIRATION_SECONDS", 60 * 60 * 24 * 30),
            ("STORES_CRYPTO_KEY_FOR_EACH_IMAGE", False),
            ("MIXED_STORAGE_FILE_STORAGE", "thumbor.storages.no_storage"),
            ("MIXED_STORAGE_CRYPTO_STORAGE", "thumbor.storages.no_storage"),
            ("MIXED_STORAGE_DETECTOR_STORAGE", "thumbor.storages.no_storage"),
            ("DETECTORS", []),
            ("FACE_DETECTOR_CASCADE_FILE", "haarcascade_frontalface_alt.xml"),
            (
                "FILTERS",
                [
                    "thumbor.filters.brightness",
                    "thumbor.filters.colorize",
                    "thumbor.filters.contrast",
                    "thumbor.filters.rgb",
                    "thumbor.filters.round_corner",
                    "thumbor.filters.quality",
                    "thumbor.filters.noise",
                    "thumbor.filters.watermark",
                    "thumbor.filters.equalize",
                    "thumbor.filters.fill",
                    "thumbor.filters.sharpen",
                    "thumbor.filters.strip_exif",
                    "thumbor.filters.strip_icc",
                    "thumbor.filters.frame",
                    "thumbor.filters.grayscale",
                    "thumbor.filters.rotate",
                    "thumbor.filters.format",
                    "thumbor.filters.max_bytes",
                    "thumbor.filters.convolution",
                    "thumbor.filters.blur",
                    "thumbor.filters.extract_focal",
                    "thumbor.filters.focal",
                    "thumbor.filters.no_upscale",
                    "thumbor.filters.saturation",
                    "thumbor.filters.max_age",
                    "thumbor.filters.curve",
                    "thumbor.filters.background_color",
                    "thumbor.filters.upscale",
                    "thumbor.filters.proportion",
                    "thumbor.filters.stretch",
                ],
            ),
        )

    def test_default_values(self):
        cfg = Config()
        for key, default_value in self.get_config():
            config_value = getattr(cfg, key)
            expect(config_value).not_to_be_null()
            expect(config_value).to_equal(default_value)

    def test_config_is_an_alias(self):
        Config.alias("OTHER_ENGINE", "ENGINE")
        cfg = Config(OTHER_ENGINE="x")
        expect(cfg.ENGINE).to_equal("x")
        expect(cfg.OTHER_ENGINE).to_equal("x")

    def test_config_is_an_aliased_key(self):
        Config.alias("LOADER_ALIAS", "LOADER")
        cfg = Config(LOADER="y")
        expect(cfg.LOADER).to_equal("y")
        expect(cfg.LOADER_ALIAS).to_equal("y")

    def test_with_aliased_aliases(self):
        Config.alias("STORAGE_ALIAS", "STORAGE")
        Config.alias("STORAGE_ALIAS_ALIAS", "STORAGE_ALIAS")
        cfg = Config(STORAGE_ALIAS_ALIAS="z")
        expect(cfg.STORAGE).to_equal("z")
        expect(cfg.STORAGE_ALIAS).to_equal("z")
        expect(cfg.STORAGE_ALIAS_ALIAS).to_equal("z")

    def test_with_aliased_aliases_with_default_values(self):
        Config.alias("STORAGE_ALIAS", "STORAGE")
        Config.alias("STORAGE_ALIAS_ALIAS", "STORAGE_ALIAS")
        cfg = Config()
        expect(cfg.STORAGE).to_equal(self.get_default_storage())
        expect(cfg.STORAGE_ALIAS).to_equal(self.get_default_storage())
        expect(cfg.STORAGE_ALIAS_ALIAS).to_equal(self.get_default_storage())
        expect(cfg.__class__.__module__).to_equal("derpconf.config")



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from typing import Any, cast

from thumbor.handler_lists import HandlerList
from thumbor.handlers.healthcheck import HealthcheckHandler


def get_handlers(context: Any) -> HandlerList:
    url = cast(str, context.config.HEALTHCHECK_ROUTE)
    return [
        (url, HealthcheckHandler),
    ]



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com


from preggy import expect
from tornado.testing import gen_test

from tests.base import FilterTestCase


class ProportionFilterTestCase(FilterTestCase):
    @gen_test
    async def test_proportion_filter_equal(self):
        image = await self.get_filtered(
            "source.jpg", "thumbor.filters.proportion", "proportion(0.60)"
        )
        expected = self.get_fixture("proportion.jpg")
        expect(image.size).to_equal(expected.size)

    @gen_test
    async def test_proportion_filter_not_equal(self):
        image = await self.get_filtered(
            "source.jpg", "thumbor.filters.proportion", "proportion(0.90)"
        )
        expected = self.get_fixture("proportion.jpg")
        expect(image.size).not_to_equal(expected.size)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com


from preggy import expect
from tornado.testing import gen_test

from tests.base import FilterTestCase


class FillFilterTestCase(FilterTestCase):
    @gen_test
    async def test_fill_filter_with_fixed_color(self):
        def config_context(context):
            context.request.fit_in = True
            context.request.width = 800
            context.request.height = 800

        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.fill",
            "fill(0000ff)",
            config_context=config_context,
        )

        expected = self.get_fixture("fill.jpg")

        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.97)

    @gen_test
    async def test_fill_filter_with_average(self):
        def config_context(context):
            context.request.fit_in = True
            context.request.width = 800
            context.request.height = 800

        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.fill",
            "fill(auto)",
            config_context=config_context,
        )

        expected = self.get_fixture("fill2.jpg")

        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.97)

    @gen_test
    async def test_fill_filter_with_full_fit_in(self):
        def config_context(context):
            context.request.fit_in = True
            context.request.full = True
            context.request.width = 800
            context.request.height = 800

        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.fill",
            "fill(auto)",
            config_context=config_context,
        )

        expected = self.get_fixture("fill3.jpg")

        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.97)

    @gen_test
    async def test_fill_filter_with_full_fit_in_orig_width(self):
        def config_context(context):
            context.request.fit_in = True
            context.request.full = True
            context.request.width = "orig"
            context.request.height = 800

        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.fill",
            "fill(auto)",
            config_context=config_context,
        )

        expected = self.get_fixture("fill3.jpg")

        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.97)

    @gen_test
    async def test_fill_filter_with_blur(self):
        def config_context(context):
            context.request.fit_in = True
            context.request.width = 800
            context.request.height = 800

        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.fill",
            "fill(blur)",
            config_context=config_context,
        )

        expected = self.get_fixture("fill4.jpg")

        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.97)

    @gen_test
    async def test_fill_filter_with_blur_transparency_true(self):
        def config_context(context):
            context.request.fit_in = True
            context.request.width = 300
            context.request.height = 325

        image = await self.get_filtered(
            "PNG_transparency_demonstration_1.png",
            "thumbor.filters.fill",
            "fill(blur,true)",
            config_context=config_context,
            mode="RGBA",
        )

        expected = self.get_fixture("fill5.png", mode="RGBA")

        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.98)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

import tempfile
from os.path import expanduser, join

from derpconf import config
from derpconf.config import Config

from thumbor import __version__
from thumbor.filters import BUILTIN_FILTERS
from thumbor.handler_lists import BUILTIN_HANDLERS

HOME = expanduser("~")

Config.define(
    "THUMBOR_LOG_CONFIG", None, "Logging configuration as json", "Logging"
)
Config.define(
    "THUMBOR_LOG_FORMAT",
    "%(asctime)s %(name)s:%(levelname)s %(message)s",
    "Log Format to be used by thumbor when writing log messages.",
    "Logging",
)

Config.define(
    "THUMBOR_LOG_DATE_FORMAT",
    "%Y-%m-%d %H:%M:%S",
    "Date Format to be used by thumbor when writing log messages.",
    "Logging",
)

Config.define(
    "MAX_WIDTH",
    0,
    "Max width in pixels for images read or generated by thumbor",
    "Imaging",
)
Config.define(
    "MAX_HEIGHT",
    0,
    "Max height in pixels for images read or generated by thumbor",
    "Imaging",
)
Config.define(
    "MAX_PIXELS", 75e6, "Max pixel count for images read by thumbor", "Imaging"
)
Config.define(
    "MIN_WIDTH",
    1,
    "Min width in pixels for images read or generated by thumbor",
    "Imaging",
)
Config.define(
    "MIN_HEIGHT",
    1,
    "Min width in pixels for images read or generated by thumbor",
    "Imaging",
)
Config.define(
    "ALLOWED_SOURCES",
    [],
    "Allowed domains for the http loader to download. These are regular expressions.",
    "Imaging",
)
Config.define(
    "QUALITY", 80, "Quality index used for generated JPEG images", "Imaging"
)
Config.define(
    "PROGRESSIVE_JPEG",
    True,
    "Exports JPEG images with the `progressive` flag set.",
    "Imaging",
)

Config.define(
    "PILLOW_JPEG_SUBSAMPLING",
    None,
    "Specify subsampling behavior for Pillow (see `subsampling` \
              in http://pillow.readthedocs.org/en/latest/handbook/image-file-formats.html#jpeg)."
    'Be careful to use int for 0,1,2 and string for "4:4:4" notation. '
    "Will ignore `quality`. Using `keep` will copy the original file's subsampling.",
    "Imaging",
)

Config.define(
    "PILLOW_JPEG_QTABLES",
    None,
    "Specify quantization tables for Pillow (see `qtables` \
              in http://pillow.readthedocs.org/en/latest/handbook/image-file-formats.html#jpeg). "
    "Will ignore `quality`. Using `keep` will copy the original file's qtables.",
    "Imaging",
)

Config.define(
    "PILLOW_RESAMPLING_FILTER",
    "LANCZOS",
    "Specify resampling filter for Pillow resize method. "
    "One of LANCZOS, NEAREST, BILINEAR, BICUBIC, HAMMING (Pillow>=3.4.0).",
    "Imaging",
)

Config.define(
    "WEBP_QUALITY",
    None,
    "Quality index used for generated WebP images. If not set (None) the same level of "
    "JPEG quality will be used. If 100 the `lossless` flag will be used.",
    "Imaging",
)

Config.define(
    "AVIF_QUALITY",
    None,
    "Quality index used for generated AVIF images. If not set (None) the same "
    "level of JPEG quality will be used. Controls the max quantizer setting, "
    "with quality=0 corresponding to max quantizer 63, and quality=100 to "
    "max quantizer=0",
    "Imaging",
)

Config.define(
    "AVIF_SPEED",
    None,
    "Quality/speed trade-off (0=slower-better, 10=fastest).",
    "Imaging",
)

Config.define(
    "AVIF_CODEC",
    "auto",
    "Codec to use for encoding AVIF images. Can be `auto`, `aom`, `rav1e`, "
    "or `svt`, depending on what is compiled into libavif.",
    "Imaging",
)

Config.define(
    "AVIF_CODEC_FALLBACK",
    "auto",
    "Codec to use for encoding AVIF images if `AVIF_CODEC` cannot be used. "
    "For instance, if AVIF_CODEC is `svt` but an image is too small or too "
    "large to be encoded with that codec, the image will be encoded using "
    "the codec from this setting instead.",
    "Imaging",
)

Config.define(
    "HEIF_QUALITY",
    None,
    "Quality index used for generated HEIF images. If not set (None) the same "
    "level of JPEG quality will be used.",
    "Imaging",
)

Config.define(
    "SRGB_PROFILE",
    None,
    "Path to the file containing the sRGB ICC profile to use when images need "
    "to be converted from other color spaces. If None, uses the default SRGB "
    "profile from Pillow ImageCMS.",
    "Imaging",
)

Config.define(
    "PNG_COMPRESSION_LEVEL",
    6,
    "Compression level for generated PNG images.",
    "Imaging",
)
Config.define(
    "PILLOW_PRESERVE_INDEXED_MODE",
    True,
    "Indicates if final image should preserve indexed mode (P or 1) of original image",
    "Imaging",
)
Config.define(
    "AUTO_WEBP",
    False,
    "Specifies whether WebP format should be used automatically if the request accepts it "
    "(via Accept header)",
    "Imaging",
)
Config.define(
    "AUTO_PNG_TO_JPG",
    False,
    "Specifies whether a PNG image should be used automatically if the png image has "
    "no transparency (via alpha layer). "
    "WARNING: Depending on case, this is not a good deal. "
    "This transformation maybe causes distortions or the size of image can increase. "
    "Images with texts, for example, the result image maybe will be distorted. "
    "Dark images, for example, the size of result image maybe will be bigger. "
    "You have to evaluate the majority of your use cases "
    "to take a decision about the usage of this conf.",
    "Imaging",
)
Config.define(
    "SVG_DPI",
    150,
    "Specify the ratio between 1in and 1px for SVG images. This is only used when "
    "rasterizing SVG images having their size units in cm or inches.",
    "Imaging",
)
Config.define(
    "MAX_AGE",
    24 * 60 * 60,
    "Max AGE sent as a header for the image served by thumbor in seconds",
    "Imaging",
)
Config.define(
    "MAX_AGE_TEMP_IMAGE",
    0,
    "Indicates the Max AGE header in seconds for "
    "temporary images (images with failed smart detection)",
    "Imaging",
)
Config.define(
    "RESPECT_ORIENTATION",
    False,
    "Indicates whether thumbor should rotate images that have an Orientation EXIF header",
    "Imaging",
)
Config.define(
    "IGNORE_SMART_ERRORS",
    False,
    "Ignore errors during smart detections and return image "
    "as a temp image (not saved in result storage and with MAX_AGE_TEMP_IMAGE age)",
    "Imaging",
)

Config.define(
    "SEND_IF_MODIFIED_LAST_MODIFIED_HEADERS",
    False,
    "Sends If-Modified-Since & Last-Modified headers; requires support from result storage",
    "Imaging",
)

Config.define(
    "PRESERVE_EXIF_INFO",
    False,
    "Preserves exif information in generated images. "
    "Increases image size in kbytes, use with caution.",
    "Imaging",
)

Config.define(
    "PRESERVE_EXIF_COPYRIGHT_INFO",
    False,
    "Preserves Exif copyright information in generated images.",
    "Imaging",
)

Config.define(
    "ALLOW_ANIMATED_GIFS",
    True,
    "Indicates whether thumbor should enable the EXPERIMENTAL support for animated gifs.",
    "Imaging",
)

Config.define(
    "USE_GIFSICLE_ENGINE",
    False,
    "Indicates whether thumbor should use gifsicle engine. "
    "Please note that smart cropping and filters are not "
    "supported for gifs using gifsicle (but won't give an error).",
    "Imaging",
)

Config.define(
    "USE_BLACKLIST",
    False,
    "Indicates whether thumbor should enable blacklist "
    "functionality to prevent processing certain images.",
    "Imaging",
)

Config.define(
    "ENGINE_THREADPOOL_SIZE",
    0,
    "Size of the thread pool used for image transformations."
    " The default value is 0 (don't use a threadpoool). "
    "Increase this if you are seeing your IOLoop getting "
    "blocked (often indicated by your upstream HTTP "
    "requests timing out)",
    "Imaging",
)

Config.define(
    "METRICS",
    "thumbor.metrics.logger_metrics",
    "The metrics backend thumbor should use to measure internal actions."
    " This must be the full name of a python module "
    "(python must be able to import it)",
    "Extensibility",
)
Config.define(
    "LOADER",
    "thumbor.loaders.http_loader",
    "The loader thumbor should use to load the original image. "
    "This must be the full name of a python module "
    "(python must be able to import it)",
    "Extensibility",
)
Config.define(
    "STORAGE",
    "thumbor.storages.file_storage",
    "The file storage thumbor should use to store original images."
    " This must be the full name of a python module "
    "(python must be able to import it)",
    "Extensibility",
)
Config.define(
    "RESULT_STORAGE",
    None,
    "The result storage thumbor should use to store generated "
    "images. This must be the full name of a python "
    "module (python must be able to import it)",
    "Extensibility",
)
Config.define(
    "ENGINE",
    "thumbor.engines.pil",
    "The imaging engine thumbor should use to perform image "
    "operations. This must be the full name of a "
    "python module (python must be able to import it)",
    "Extensibility",
)

Config.define(
    "GIF_ENGINE",
    "thumbor.engines.gif",
    "The gif engine thumbor should use to perform image operations."
    " This must be the full name of a "
    "python module (python must be able to import it)",
    "Extensibility",
)

Config.define(
    "SECURITY_KEY",
    "MY_SECURE_KEY",
    "The security key thumbor uses to sign image URLs",
    "Security",
)

Config.define(
    "ALLOW_UNSAFE_URL",
    True,
    "Indicates if the /unsafe URL should be available",
    "Security",
)
Config.define(
    "ENABLE_ETAGS", True, "Enables automatically generated etags", "HTTP"
)
Config.define(
    "MAX_ID_LENGTH",
    32,
    "Set maximum id length for images when stored",
    "Storage",
)
Config.define(
    "GC_INTERVAL",
    None,
    "Set garbage collection interval in seconds",
    "Performance",
)

Config.define(
    "HEALTHCHECK_ROUTE", r"/healthcheck/?", "Healthcheck route.", "Healthcheck"
)

# METRICS OPTIONS
Config.define(
    "STATSD_HOST", None, "Host to send statsd instrumentation to", "Metrics"
)
Config.define(
    "STATSD_PORT", 8125, "Port to send statsd instrumentation to", "Metrics"
)
Config.define("STATSD_PREFIX", None, "Prefix for statsd", "Metrics")

# FILE LOADER OPTIONS
Config.define(
    "FILE_LOADER_ROOT_PATH",
    HOME,
    "The root path where the File Loader will try to find images",
    "File Loader",
)

# HTTP LOADER OPTIONS
Config.define(
    "HTTP_LOADER_CONNECT_TIMEOUT",
    5,
    "The maximum number of seconds libcurl can take to connect to an image being loaded",
    "HTTP Loader",
)
Config.define(
    "HTTP_LOADER_REQUEST_TIMEOUT",
    20,
    "The maximum number of seconds libcurl can take to download an image",
    "HTTP Loader",
)
Config.define(
    "HTTP_LOADER_FOLLOW_REDIRECTS",
    True,
    "Indicates whether libcurl should follow redirects when downloading an image",
    "HTTP Loader",
)
Config.define(
    "HTTP_LOADER_MAX_REDIRECTS",
    5,
    "Indicates the number of redirects libcurl should follow when downloading an image",
    "HTTP Loader",
)
Config.define(
    "HTTP_LOADER_MAX_CLIENTS",
    10,
    "The maximum number of simultaneous HTTP connections the loader can make before queuing",
    "HTTP Loader",
)
Config.define(
    "HTTP_LOADER_FORWARD_USER_AGENT",
    False,
    "Indicates whether thumbor should forward the user agent of the requesting user",
    "HTTP Loader",
)
Config.define(
    "HTTP_LOADER_FORWARD_ALL_HEADERS",
    False,
    "Indicates whether thumbor should forward the headers of the request",
    "HTTP Loader",
)
Config.define(
    "HTTP_LOADER_FORWARD_HEADERS_WHITELIST",
    [],
    "Indicates which headers should be forwarded among all the headers of the request",
    "HTTP Loader",
)
Config.define(
    "HTTP_LOADER_DEFAULT_USER_AGENT",
    f"Thumbor/{__version__}",
    "Default user agent for thumbor http loader requests",
    "HTTP Loader",
)
Config.define(
    "HTTP_LOADER_PROXY_HOST",
    None,
    "The proxy host needed to load images through",
    "HTTP Loader",
)
Config.define(
    "HTTP_LOADER_PROXY_PORT",
    None,
    "The proxy port for the proxy host",
    "HTTP Loader",
)
Config.define(
    "HTTP_LOADER_PROXY_USERNAME",
    None,
    "The proxy username for the proxy host",
    "HTTP Loader",
)
Config.define(
    "HTTP_LOADER_PROXY_PASSWORD",
    None,
    "The proxy password for the proxy host",
    "HTTP Loader",
)
Config.define(
    "HTTP_LOADER_CA_CERTS",
    None,
    "The filename of CA certificates in PEM format",
    "HTTP Loader",
)
Config.define(
    "HTTP_LOADER_VALIDATE_CERTS",
    None,
    "Validate the server’s certificate for HTTPS requests",
    "HTTP Loader",
)
Config.define(
    "HTTP_LOADER_CLIENT_KEY",
    None,
    "The filename for client SSL key",
    "HTTP Loader",
)
Config.define(
    "HTTP_LOADER_CLIENT_CERT",
    None,
    "The filename for client SSL certificate",
    "HTTP Loader",
)
Config.define(
    "HTTP_LOADER_CURL_ASYNC_HTTP_CLIENT",
    False,
    "If the CurlAsyncHTTPClient should be used",
    "HTTP Loader",
)
Config.define(
    "HTTP_LOADER_CURL_LOW_SPEED_TIME",
    0,
    "If HTTP_LOADER_CURL_LOW_SPEED_LIMIT and HTTP_LOADER_CURL_ASYNC_HTTP_CLIENT "
    + "are set, then this is the time in seconds as integer after a download should "
    + "timeout if the speed is below HTTP_LOADER_CURL_LOW_SPEED_LIMIT for that long",
)
Config.define(
    "HTTP_LOADER_CURL_LOW_SPEED_LIMIT",
    0,
    "If HTTP_LOADER_CURL_LOW_SPEED_TIME and HTTP_LOADER_CURL_ASYNC_HTTP_CLIENT "
    + "are set, then this is the limit in bytes per second as integer which should "
    + "timeout if the speed is below that limit for HTTP_LOADER_CURL_LOW_SPEED_TIME seconds",
)

# FILE STORAGE GENERIC OPTIONS
Config.define(
    "STORAGE_EXPIRATION_SECONDS",
    60 * 60 * 24 * 30,
    "Expiration in seconds for the images in the File Storage. Defaults to one month",
    "File Storage",
)
Config.define(
    "STORES_CRYPTO_KEY_FOR_EACH_IMAGE",
    False,
    "Indicates whether thumbor should store the signing key for each image in the file storage. "
    + "This allows the key to be changed and old images to still be properly found",
    "File Storage",
)

# FILE STORAGE OPTIONS
Config.define(
    "FILE_STORAGE_ROOT_PATH",
    join(tempfile.gettempdir(), "thumbor", "storage"),
    "The root path where the File Storage will try to find images",
    "File Storage",
)

# PHOTO UPLOAD OPTIONS
Config.define(
    "UPLOAD_MAX_SIZE",
    0,
    "Max size in bytes for images uploaded to thumbor",
    "Upload",
)
Config.define(
    "UPLOAD_ENABLED",
    False,
    "Indicates whether thumbor should enable File uploads",
    "Upload",
)
Config.define(
    "UPLOAD_PHOTO_STORAGE",
    "thumbor.storages.file_storage",
    "The type of storage to store uploaded images with",
    "Upload",
)
Config.define(
    "UPLOAD_DELETE_ALLOWED",
    False,
    "Indicates whether image deletion should be allowed",
    "Upload",
)
Config.define(
    "UPLOAD_PUT_ALLOWED",
    False,
    "Indicates whether image overwrite should be allowed",
    "Upload",
)
Config.define(
    "UPLOAD_DEFAULT_FILENAME",
    "image",
    "Default filename for image uploaded",
    "Upload",
)

# ALIASES FOR OLD PHOTO UPLOAD OPTIONS
Config.alias("MAX_SIZE", "UPLOAD_MAX_SIZE")
Config.alias("ENABLE_ORIGINAL_PHOTO_UPLOAD", "UPLOAD_ENABLED")
Config.alias("ORIGINAL_PHOTO_STORAGE", "UPLOAD_PHOTO_STORAGE")
Config.alias("ALLOW_ORIGINAL_PHOTO_DELETION", "UPLOAD_DELETE_ALLOWED")
Config.alias("ALLOW_ORIGINAL_PHOTO_PUTTING", "UPLOAD_PUT_ALLOWED")

# MIXED STORAGE OPTIONS
Config.define(
    "MIXED_STORAGE_FILE_STORAGE",
    "thumbor.storages.no_storage",
    "Mixed Storage file storage. This must be the full name "
    "of a python module (python must be able to import it)",
    "Mixed Storage",
)
Config.define(
    "MIXED_STORAGE_CRYPTO_STORAGE",
    "thumbor.storages.no_storage",
    "Mixed Storage signing key storage. This must be the full "
    "name of a python module (python must be able to import it)",
    "Mixed Storage",
)
Config.define(
    "MIXED_STORAGE_DETECTOR_STORAGE",
    "thumbor.storages.no_storage",
    "Mixed Storage detector information storage. This must be the full "
    "name of a python module (python must be able to import it)",
    "Mixed Storage",
)

# JSON META ENGINE OPTIONS
Config.define(
    "META_CALLBACK_NAME",
    None,
    "The callback function name that should be used by the META route for JSONP access",
    "Meta",
)

# DETECTORS OPTIONS
Config.define(
    "DETECTORS",
    [],
    "List of detectors that thumbor should use to find faces and/or features. All of them must be "
    + "full names of python modules (python must be able to import it)",
    "Detection",
)

# FACE DETECTOR CASCADE FILE
Config.define(
    "FACE_DETECTOR_CASCADE_FILE",
    "haarcascade_frontalface_alt.xml",
    "The cascade file that opencv will use to detect faces.",
    "Detection",
)

Config.define(
    "GLASSES_DETECTOR_CASCADE_FILE",
    "haarcascade_eye_tree_eyeglasses.xml",
    "The cascade file that opencv will use to detect glasses.",
    "Detection",
)

Config.define(
    "PROFILE_DETECTOR_CASCADE_FILE",
    "haarcascade_profileface.xml",
    "The cascade file that opencv will use to detect profile faces.",
    "Detection",
)

Config.define(
    "OPTIMIZERS",
    [
        # 'thumbor.optimizers.jpegtran',
        # 'thumbor_plugins.optimizers.gifv',
    ],
    "List of optimizers that thumbor will use to optimize images",
    "Optimizers",
)

# OPTIMIZER CONFIGURATIONS
Config.define(
    "JPEGTRAN_PATH",
    "/usr/bin/jpegtran",
    "Path for the jpegtran binary",
    "Optimizers",
)

Config.define(
    "JPEGTRAN_SCANS_FILE",
    "",
    "Path for the progressive scans file to use with "
    "jpegtran optimizer. Implies progressive jpeg output",
    "Optimizers",
)

Config.define(
    "FFMPEG_PATH",
    "/usr/local/bin/ffmpeg",
    "Path for the ffmpeg binary used to generate gifv(h.264)",
    "Optimizers",
)

# AVAILABLE FILTERS
Config.define(
    "FILTERS",
    BUILTIN_FILTERS,
    "List of filters that thumbor will allow to be used in generated images. All of them must be "
    + "full names of python modules (python must be able to import it)",
    "Filters",
)

# RESULT STORAGE
Config.define(
    "RESULT_STORAGE_EXPIRATION_SECONDS",
    0,
    "Expiration in seconds of generated images in the result storage",
    "Result Storage",
)  # Never expires
Config.define(
    "RESULT_STORAGE_FILE_STORAGE_ROOT_PATH",
    join(tempfile.gettempdir(), "thumbor", "result_storage"),
    "Path where the Result storage will store generated images",
    "Result Storage",
)
Config.define(
    "RESULT_STORAGE_STORES_UNSAFE",
    False,
    "Indicates whether unsafe requests should also be stored in the Result Storage",
    "Result Storage",
)

# QUEUED DETECTOR REDIS OPTIONS
Config.define(
    "REDIS_QUEUE_SERVER_HOST",
    "localhost",
    "Server host for the queued redis detector",
    "Queued Redis Detector",
)
Config.define(
    "REDIS_QUEUE_SERVER_PORT",
    6379,
    "Server port for the queued redis detector",
    "Queued Redis Detector",
)
Config.define(
    "REDIS_QUEUE_SERVER_DB",
    0,
    "Server database index for the queued redis detector",
    "Queued Redis Detector",
)
Config.define(
    "REDIS_QUEUE_SERVER_PASSWORD",
    None,
    "Server password for the queued redis detector",
    "Queued Redis Detector",
)
Config.define(
    "REDIS_QUEUE_MODE",
    "single_node",
    "Redis operation mode 'single_node' or 'sentinel'",
    "Queued Redis Detector",
)

# QUEUED DETECTOR REDIS SENTINEL OPTIONS
Config.define(
    "REDIS_QUEUE_SENTINEL_INSTANCES",
    "localhost:26379",
    "Sentinel server instances for the queued redis detector",
    "Queued Redis Detector",
)
Config.define(
    "REDIS_QUEUE_SENTINEL_PASSWORD",
    None,
    "Sentinel server password for the queued redis detector",
    "Queued Redis Detector",
)
Config.define(
    "REDIS_QUEUE_SENTINEL_MASTER_INSTANCE",
    "master",
    "Sentinel server master instance for the queued redis detector",
    "Queued Redis Detector",
)
Config.define(
    "REDIS_QUEUE_SENTINEL_MASTER_PASSWORD",
    None,
    "Sentinel server master password for the queued redis detector",
    "Queued Redis Detector",
)
Config.define(
    "REDIS_QUEUE_SENTINEL_MASTER_DB",
    0,
    "Sentinel server master database index for the queued redis detector",
    "Queued Redis Detector",
)
Config.define(
    "REDIS_QUEUE_SENTINEL_SOCKET_TIMEOUT",
    10.0,
    "Sentinel server socket timeout for the queued redis detector",
    "Queued Redis Detector",
)

# QUEUED DETECTOR SQS OPTIONS
Config.define("SQS_QUEUE_KEY_ID", None, "AWS key id", "Queued SQS Detector")
Config.define(
    "SQS_QUEUE_KEY_SECRET", None, "AWS key secret", "Queued SQS Detector"
)
Config.define(
    "SQS_QUEUE_REGION", "us-east-1", "AWS SQS region", "Queued SQS Detector"
)

# ERROR HANDLING
Config.define(
    "USE_CUSTOM_ERROR_HANDLING",
    False,
    "This configuration indicates whether thumbor should use a custom error handler.",
    "Errors",
)
Config.define(
    "ERROR_HANDLER_MODULE",
    "thumbor.error_handlers.sentry",
    "Error reporting module. Needs to contain a class called ErrorHandler with a "
    + "handle_error(context, handler, exception) method.",
    "Errors",
)

# SENTRY REPORTING MODULE
Config.define(
    "SENTRY_DSN_URL",
    "",
    "Sentry thumbor project dsn. i.e.: "
    + "http://5a63d58ae7b94f1dab3dee740b301d6a:73eea45d3e8649239a973087e8f21f98@localhost:9000/2",
    "Errors - Sentry",
)

# SENTRY REPORTING MODULE
Config.define(
    "SENTRY_ENVIRONMENT",
    None,
    "Sentry environment i.e.: staging ",
    "Errors - Sentry",
)

# FILE REPORTING MODULE
Config.define("ERROR_FILE_LOGGER", None, "File of error log as json", "Errors")
Config.define(
    "ERROR_FILE_NAME_USE_CONTEXT",
    False,
    "File of error log name is parametrized with context attribute",
    "Errors",
)

# SIGNER MODULE
Config.define(
    "URL_SIGNER",
    "libthumbor.url_signers.base64_hmac_sha1",
    "The url signer thumbor should use to verify url signatures."
    + "This must be the full name of a python module "
    + "(python must be able to import it)",
    "Extensibility",
)

# SERVER
Config.define(
    "MAX_WAIT_SECONDS_BEFORE_SERVER_SHUTDOWN",
    0,
    "The amount of time to wait before shutting down the server, i.e. stop accepting requests.",
    "Server",
)
Config.define(
    "MAX_WAIT_SECONDS_BEFORE_IO_SHUTDOWN",
    0,
    "The amount of time to wait before shutting down all io, after the server has been stopped",
    "Server",
)
Config.define(
    "NON_BLOCKING_SOCKETS",
    False,
    "If True, thumbor will ensure that the socket from the file descriptor number passed using"
    " the --fd flag is non-blocking. This setting has no effect if the --fd flag is a path,"
    " sockets created that way are always non-blocking.",
)

# HANDLER LISTS
Config.define(
    "HANDLER_LISTS",
    BUILTIN_HANDLERS,
    "Handler Lists are responsible for adding new handlers to thumbor app.",
    "HandlerLists",
)


Config.define(
    "APP_CLASS",
    "thumbor.app.ThumborServiceApp",
    "Custom app class to override ThumborServiceApp. "
    "This config value is overridden by the -a command-line parameter.",
)

Config.define(
    "ACCESS_CONTROL_ALLOW_ORIGIN_HEADER",
    False,
    "Sends Access-Control-Allow-Origin header",
)


# COMPATIBILITY

Config.define(
    "COMPATIBILITY_LEGACY_LOADER",
    None,
    "Loader that will be used with the compatibility layer, instead of the "
    "compatibility loader. Please only use this if you can't use up-to-date loaders.",
    "Compatibility",
)

Config.define(
    "COMPATIBILITY_LEGACY_STORAGE",
    None,
    "Storage that will be used with the compatibility layer, instead of the "
    "compatibility storage. Please only use this if you can't use up-to-date storages.",
    "Compatibility",
)

Config.define(
    "COMPATIBILITY_LEGACY_RESULT_STORAGE",
    None,
    "Result Storage that will be used with the compatibility layer, instead of the "
    "compatibility result storage. Please only use this if you can't use "
    "up-to-date result storages.",
    "Compatibility",
)


def generate_config():
    config.generate_config()


def format_value(value):
    if isinstance(value, str):
        return f"'{value}'"
    if isinstance(value, (tuple, list, set)):
        representation = "[\n"
        for item in value:
            representation += f"#    {item}"
        representation += "#]"
        return representation
    return value


if __name__ == "__main__":
    generate_config()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

import shutil
import tempfile
from os.path import abspath, dirname, join

from preggy import expect
from tornado.testing import gen_test

from tests.base import TestCase
from thumbor.config import Config
from thumbor.context import ServerParameters
from thumbor.importer import Importer
from thumbor.server import validate_config

# pylint: disable=broad-except,abstract-method,attribute-defined-outside-init,line-too-long,too-many-public-methods
# pylint: disable=too-many-lines


class ImageBadRequestDecompressionBomb(TestCase):
    def setUp(self):
        self.root_path = tempfile.mkdtemp()
        self.loader_path = abspath(
            join(dirname(__file__), "../fixtures/images/")
        )
        self.base_uri = "/image"
        super().setUp()

    def tearDown(self):
        shutil.rmtree(self.root_path)

    async def get_as_webp(self, url):
        return await self.async_fetch(
            url, headers={"Accept": "image/webp,*/*;q=0.8"}
        )

    def get_config(self):
        cfg = Config(SECURITY_KEY="ACME-SEC")
        cfg.LOADER = "thumbor.loaders.file_loader"
        cfg.FILE_LOADER_ROOT_PATH = self.loader_path
        cfg.STORAGE = "thumbor.storages.no_storage"
        cfg.AUTO_WEBP = True
        return cfg

    def get_server(self):
        server = ServerParameters(
            8889, "localhost", "thumbor.conf", None, "info", None
        )
        validate_config(self.config, server)
        return server

    def get_importer(self):
        importer = Importer(self.config)
        importer.import_modules()
        return importer

    @gen_test
    async def test_should_bad_request_if_bigger_than_75_megapixels(self):
        response = await self.get_as_webp("/unsafe/16384x16384.png")
        expect(response.code).to_equal(400)

    @gen_test
    async def test_should_bad_request_if_bigger_than_75_megapixels_jpeg(self):
        response = await self.get_as_webp("/unsafe/9643x10328.jpg")
        expect(response.code).to_equal(400)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

import logging
from shutil import which
from unittest import TestCase, mock

from preggy import expect
import pytest
from PIL import Image, ImageCms

from thumbor.utils import (
    CONTENT_TYPE,
    EXTENSION,
    deprecated,
    logger,
    ensure_srgb,
    get_color_space,
)


class UtilsTestCase(TestCase):
    def setUp(self):
        self.handled = False
        super().setUp()

    @staticmethod
    def test_can_get_content_type():
        expect(CONTENT_TYPE.get(".jpg")).to_equal("image/jpeg")
        expect(CONTENT_TYPE.get(".jpeg")).to_equal("image/jpeg")
        expect(CONTENT_TYPE.get(".gif")).to_equal("image/gif")
        expect(CONTENT_TYPE.get(".png")).to_equal("image/png")
        expect(CONTENT_TYPE.get(".webp")).to_equal("image/webp")
        expect(CONTENT_TYPE.get(".mp4")).to_equal("video/mp4")
        expect(CONTENT_TYPE.get(".webm")).to_equal("video/webm")
        expect(CONTENT_TYPE.get(".svg")).to_equal("image/svg+xml")
        expect(CONTENT_TYPE.get(".tif")).to_equal("image/tiff")
        expect(CONTENT_TYPE.get(".tiff")).to_equal("image/tiff")
        expect(CONTENT_TYPE.get(".avif")).to_equal("image/avif")
        expect(CONTENT_TYPE.get(".heic")).to_equal("image/heif")
        expect(CONTENT_TYPE.get(".heif")).to_equal("image/heif")

    @staticmethod
    def test_can_get_extension():
        expect(EXTENSION.get("image/jpeg")).to_equal(".jpg")
        expect(EXTENSION.get("image/gif")).to_equal(".gif")
        expect(EXTENSION.get("image/png")).to_equal(".png")
        expect(EXTENSION.get("image/webp")).to_equal(".webp")
        expect(EXTENSION.get("video/mp4")).to_equal(".mp4")
        expect(EXTENSION.get("video/webm")).to_equal(".webm")
        expect(EXTENSION.get("image/svg+xml")).to_equal(".svg")
        expect(EXTENSION.get("image/tiff")).to_equal(".tif")
        expect(EXTENSION.get("image/avif")).to_equal(".avif")
        expect(EXTENSION.get("image/heif")).to_equal(".heic")

    @staticmethod
    def test_can_get_logger():
        expect(logger.name).to_equal("thumbor")

    @staticmethod
    def test_deprecated_logs_msg():
        @deprecated("func2")
        def test_func():
            pass

        with mock.patch.object(logger, "warning") as mock_warn:
            test_func()
            mock_warn.assert_called_once_with(
                "Deprecated function %s%s", "test_func", "func2"
            )

    @staticmethod
    def test_can_which_by_path():
        result = which("/bin/ls")
        expect(result).to_include("/bin/ls")

        result = which("/tmp")
        expect(result).to_be_null()

    @staticmethod
    def test_can_which_by_env():
        result = which("ls")
        expect(result).to_include("/bin/ls")

        result = which("invalid-command")
        expect(result).to_be_null()

    @staticmethod
    def test_logger_should_be_instance_of_python_logger():
        expect(logger).to_be_instance_of(logging.Logger)

    @staticmethod
    def test_logger_should_not_be_null():
        expect(logger).not_to_be_null()

    @staticmethod
    def test_logger_should_not_be_an_error():
        expect(logger).not_to_be_an_error()


def test_get_color_space_handles_invalid_icc():
    img = Image.new("RGB", (50, 50), "white")
    img.info["icc_profile"] = b"invalid"
    result = get_color_space(img)
    assert result is None


def test_get_color_space_missing_imagecms_returns_none():
    with mock.patch("thumbor.utils.ImageCms", new=None):
        img = Image.open("tests/fixtures/images/cmyk-icc.jpg")
        assert get_color_space(img) is None


def test_ensure_srgb_handles_invalid_icc():
    img = Image.new("RGB", (50, 50), "white")
    img.info["icc_profile"] = b"invalid"
    retval = ensure_srgb(img)
    assert retval is None


def test_ensure_srgb_missing_imagecms_runtimeerror():
    with mock.patch("thumbor.utils.ImageCms", new=None):
        img = Image.open("tests/fixtures/images/cmyk-icc.jpg")
        with pytest.raises(RuntimeError):
            ensure_srgb(img)


def test_ensure_srgb_keeps_existing_srgb_profile():
    img = Image.new("RGB", (50, 50), "white")
    profile = ImageCms.ImageCmsProfile(ImageCms.createProfile("sRGB"))
    img.info["icc_profile"] = profile.tobytes()
    retval = ensure_srgb(img)
    assert img is retval


@pytest.mark.parametrize("color_space", ("LAB", "XYZ"))
def test_ensure_srgb_returns_none_for_unsupported_color_space(color_space):
    profile = ImageCms.ImageCmsProfile(ImageCms.createProfile(color_space))
    img = Image.new("RGB", (50, 50), "white")
    img.info["icc_profile"] = profile.tobytes()
    retval = ensure_srgb(img)
    assert retval is None


def test_ensure_srgb_returns_none_for_animated_cmyk():
    img = Image.open("tests/fixtures/images/cmyk-icc.jpg")
    with mock.patch.object(img, "is_animated", new=True, create=True):
        retval = ensure_srgb(img)
        assert retval is None



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from os.path import abspath
from unittest import mock

from preggy import expect
from tornado.testing import gen_test

from tests.base import DetectorTestCase
from thumbor.detectors.profile_detector import Detector


class ProfileDetectorTestCase(DetectorTestCase):
    @gen_test
    async def test_detector_uses_proper_cascade(self):
        cascade = "./tests/fixtures/haarcascade_profileface.xml"
        ctx = mock.Mock(
            config=mock.Mock(
                PROFILE_DETECTOR_CASCADE_FILE=abspath(cascade),
            )
        )

        detector = Detector(ctx, 1, [])
        expect(detector).not_to_be_null()



# #!/usr/bin/python
# # -*- coding: utf-8 -*-

# # thumbor imaging service
# # https://github.com/thumbor/thumbor/wiki

# # Licensed under the MIT license:
# # http://www.opensource.org/licenses/mit-license
# # Copyright (c) 2011 globo.com thumbor@googlegroups.com

from preggy import expect

from tests.base import TestCase
from thumbor.point import FocalPoint


class FocalPointTestCase(TestCase):
    def test_default_alignment_percentages(self):
        percentages = FocalPoint.ALIGNMENT_PERCENTAGES
        expect(percentages["left"]).to_equal(0.0)
        expect(percentages["center"]).to_equal(0.5)
        expect(percentages["right"]).to_equal(1.0)
        expect(percentages["top"]).to_equal(0.0)
        expect(percentages["middle"]).to_equal(0.5)
        expect(percentages["bottom"]).to_equal(1.0)

    def test_new_point_default_weight(self):
        point = FocalPoint(10, 20)
        expect(point.x).to_equal(10)
        expect(point.y).to_equal(20)

    def test_new_point_weighted(self):
        point = FocalPoint(x=10, y=20, height=1.0, width=3.0, weight=3.0)
        expect(point.weight).to_equal(3.0)
        expect(str(point)).to_equal(
            "FocalPoint(x: 10, y: 20, width: 3, height: 1, weight: 3, origin: alignment)"
        )

    def test_new_point_from_dict(self):
        point = FocalPoint.from_dict({"x": 10, "y": 20, "z": 5})
        expect(point.x).to_equal(10)
        expect(point.y).to_equal(20)
        expect(point.weight).to_equal(5)

    def test_new_point_to_dict(self):
        point = FocalPoint.from_dict(
            {"x": 10.1, "y": 20.1, "z": 5.1, "width": 1.1, "height": 1.6}
        )
        expect(point.to_dict()).to_be_like(
            {
                "x": 10,
                "y": 20,
                "z": 5,
                "origin": "alignment",
                "width": 1,
                "height": 1,
            }
        )

    def test_new_point_square_point(self):
        point = FocalPoint.from_square(x=350, y=50, width=110, height=110)
        expect(point.x).to_equal(405)
        expect(point.y).to_equal(105)
        expect(point.weight).to_equal(12100)

    def test_aligned_point_center_middle(self):
        point = FocalPoint.from_alignment("center", "middle", 300, 200)
        expect(point.x).to_equal(150)
        expect(point.y).to_equal(100)
        expect(point.weight).to_equal(1.0)

    def test_aligned_point_top_left(self):
        point = FocalPoint.from_alignment("left", "top", 300, 200)
        expect(point.x).to_equal(0)
        expect(point.y).to_equal(0)
        expect(point.weight).to_equal(1.0)

    def test_aligned_point_bottom_right(self):
        point = FocalPoint.from_alignment("right", "bottom", 300, 200)
        expect(point.x).to_equal(300)
        expect(point.y).to_equal(200)
        expect(point.weight).to_equal(1.0)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from thumbor.utils import logger

try:
    import cv2  # noqa
    import numpy as np  # noqa

    CV_AVAILABLE = True
except ImportError:
    CV_AVAILABLE = False


class BaseDetector:
    def __init__(self, context, index, detectors):
        self.context = context
        self.index = index
        self.detectors = detectors

    def verify_cv(self) -> bool:
        if CV_AVAILABLE:
            return True

        logger.error(
            "OpenCV (cv2) is not available for thumbor. "
            "thumbor.detectors.local_detector.CascadeLoaderDetector "
            "can't be executed. Skipping..."
        )

        return False

    async def detect(self):
        raise NotImplementedError()

    async def next(self):
        if not self.detectors or self.index > len(self.detectors) - 2:
            return

        next_detector = self.detectors[self.index + 1](
            self.context, self.index + 1, self.detectors
        )
        await next_detector.detect()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from os.path import abspath, dirname, join

from preggy import expect
from tornado.testing import gen_test

from tests.base import TestCase
from thumbor.compatibility import compatibility_get
from thumbor.config import Config
from thumbor.context import Context
from thumbor.importer import Importer
from thumbor.loaders import LoaderResult

STORAGE_PATH = abspath(join(dirname(__file__), "../fixtures/images/"))


class SimpleCompatibilityTestCase(TestCase):
    @gen_test
    async def test_can_get_result(self):
        Importer.deprecated_monkey_patch_tornado_return_future()
        from tests.compatibility.legacy_file_loader import (  # pylint: disable=import-outside-toplevel
            load,
        )

        config = Config(FILE_LOADER_ROOT_PATH=STORAGE_PATH)
        ctx = Context(config=config)
        result = await compatibility_get(ctx, "image.jpg", func=load)
        expect(result).to_be_instance_of(LoaderResult)
        with open(join(STORAGE_PATH, "image.jpg"), "rb") as img:
            expect(result.buffer).to_equal(img.read())
            expect(result.successful).to_be_true()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from thumbor.ext.filters import _fill
from thumbor.filters import BaseFilter, filter_method
from thumbor.filters.blur import apply_blur


class Filter(BaseFilter):
    def get_median_color(self):
        mode, data = self.engine.image_data_as_rgb()
        red, green, blue = _fill.apply(mode, data)
        return "%02x%02x%02x" % (  # pylint: disable=consider-using-f-string
            red,
            green,
            blue,
        )

    @filter_method(r"[\w]+", BaseFilter.Boolean)
    async def fill(self, color, fill_transparent=False):
        self.fill_engine = self.engine.__class__(self.context)
        target_width = (
            self.context.request.width
            if self.context.request.width != 0
            else self.engine.size[0]
        )
        target_height = (
            self.context.request.height
            if self.context.request.height != 0
            else self.engine.size[1]
        )

        # if the color is 'auto'
        # we will calculate the median color of
        # all the pixels in the image and return
        if color == "auto":
            color = self.get_median_color()

        if color == "blur":
            self.fill_engine.image = self.fill_engine.gen_image(
                self.engine.size, "transparent"
            )
            self.fill_engine.paste(self.engine, (0, 0))
            mode, data = self.fill_engine.image_data_as_rgb()
            data = apply_blur(mode, data, self.fill_engine.size, 50)
            self.fill_engine.set_image_data(data)
            self.fill_engine.resize(target_width, target_height)
        else:
            try:
                self.fill_engine.image = self.fill_engine.gen_image(
                    (target_width, target_height), color
                )
            except (ValueError, RuntimeError):
                self.fill_engine.image = self.fill_engine.gen_image(
                    (target_width, target_height), f"#{color}"
                )

        final_width, final_height = self.engine.size

        top_left_x = (target_width - final_width) // 2  # top left
        top_left_y = (target_height - final_height) // 2

        self.fill_engine.paste(
            self.engine, (top_left_x, top_left_y), merge=fill_transparent
        )
        self.engine.image = self.fill_engine.image



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from os.path import abspath, dirname, join

from preggy import expect
from tornado.testing import gen_test

from tests.base import TestCase
from thumbor.config import Config
from thumbor.context import Context
from thumbor.loaders import LoaderResult
from thumbor.loaders.file_loader import load

STORAGE_PATH = abspath(join(dirname(__file__), "../fixtures/images/"))


class FileLoaderTestCase(TestCase):
    def setUp(self):
        super().setUp()
        config = Config(FILE_LOADER_ROOT_PATH=STORAGE_PATH)
        self.ctx = Context(config=config)

    async def load_file(self, file_name):
        return await load(self.ctx, file_name)

    @gen_test
    async def test_should_load_file(self):
        result = await self.load_file("image.jpg")
        expect(result).to_be_instance_of(LoaderResult)
        with open(join(STORAGE_PATH, "image.jpg"), "rb") as img:
            expect(result.buffer).to_equal(img.read())
            expect(result.successful).to_be_true()

    @gen_test
    async def test_should_fail_when_inexistent_file(self):
        result = await self.load_file("image_NOT.jpg")
        expect(result).to_be_instance_of(LoaderResult)
        expect(result.buffer).to_equal(None)
        expect(result.successful).to_be_false()

    @gen_test
    async def test_should_fail_when_outside_root_path(self):
        result = await self.load_file("../__init__.py")
        expect(result).to_be_instance_of(LoaderResult)
        expect(result.buffer).to_equal(None)
        expect(result.successful).to_be_false()

    @gen_test
    async def test_should_load_file_with_spaces_in_name(self):
        result = await self.load_file("image .jpg")
        expect(result).to_be_instance_of(LoaderResult)
        with open(join(STORAGE_PATH, "image.jpg"), "rb") as img:
            expect(result.buffer).to_equal(img.read())
            expect(result.successful).to_be_true()

    @gen_test
    async def test_should_load_file_with_spaces_encoded_in_name(self):
        result = await self.load_file("image2%20.jpg")
        expect(result).to_be_instance_of(LoaderResult)
        with open(join(STORAGE_PATH, "image2%20.jpg"), "rb") as img:
            expect(result.buffer).to_equal(img.read())
            expect(result.successful).to_be_true()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from typing import Dict


class LoaderResult:

    ERROR_NOT_FOUND = "not_found"
    ERROR_UPSTREAM = "upstream"
    ERROR_TIMEOUT = "timeout"
    ERROR_BAD_REQUEST = "bad_request"

    def __init__(
        self,
        buffer: bytes = None,
        successful: bool = True,
        error: str = None,
        metadata: Dict[str, any] = None,
        extras: Dict[str, any] = None,
    ):
        """
        :param buffer: The media buffer

        :param successful: True when the media has been loaded.
        :type successful: bool

        :param error: Error code
        :type error: str

        :param metadata: Dictionary of metadata about the buffer
        :type metadata: dict

        :param extras: Dictionary of extra information about the error
        :type metadata: dict
        """

        if metadata is None:
            metadata = {}

        if extras is None:
            extras = {}

        self.buffer = buffer
        self.successful = successful
        self.error = error
        self.metadata = metadata
        self.extras = extras



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from os.path import abspath, dirname, join
from unittest.mock import patch

from preggy import expect
from tornado.testing import gen_test

import thumbor
import thumbor.loaders.file_loader_http_fallback as loader
from tests.base import TestCase
from thumbor.config import Config
from thumbor.context import Context
from thumbor.loaders import LoaderResult

STORAGE_PATH = abspath(join(dirname(__file__), "../fixtures/images/"))


async def dummy_file_load(
    context, url, normalize_url_func=None
):  # pylint: disable=unused-argument
    result = LoaderResult(
        successful=True,
        buffer="file",
    )

    return result


async def dummy_http_load(
    context, url, normalize_url_func=None
):  # pylint: disable=unused-argument
    result = LoaderResult(
        successful=True,
        buffer="http",
    )

    return result


class FileLoaderHttpFallbackFileTestCase(TestCase):
    def setUp(self):
        super().setUp()
        config = Config(FILE_LOADER_ROOT_PATH=STORAGE_PATH)
        self.ctx = Context(config=config)

    @patch.object(thumbor.loaders.file_loader, "load", dummy_file_load)
    @gen_test
    async def test_should_load_file(self):
        result = await loader.load(self.ctx, "image.jpg")
        expect(result).to_be_instance_of(LoaderResult)
        expect(result.buffer).to_equal("file")


class FileLoaderHttpFallbackHttpTestCase(TestCase):
    @patch.object(thumbor.loaders.http_loader, "load", dummy_http_load)
    @gen_test
    async def test_should_load_http(self):
        url = self.get_url("http:/www.google.com/example_image.png")
        config = Config()
        ctx = Context(None, config, None)

        result = await loader.load(ctx, url)

        expect(result).to_be_instance_of(LoaderResult)
        expect(result.buffer).to_equal("http")

    @patch.object(thumbor.loaders.http_loader, "load", dummy_http_load)
    @gen_test
    async def test_should_fail_with_disallowed_origin(self):
        url = "http:/www.google.com/example_image.png"
        config = Config(ALLOWED_SOURCES=[".+.domain1.com"])
        ctx = Context(None, config, None)

        result = await loader.load(ctx, url)

        expect(result).to_be_instance_of(LoaderResult)
        expect(result.successful).to_be_false()
        expect(result.error).to_equal(LoaderResult.ERROR_BAD_REQUEST)
        expect(result.extras["reason"]).to_equal("Unallowed domain")
        expect(result.extras["source"]).to_equal(url)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

import os
import subprocess
import tempfile
from shutil import which

import pytest
from preggy import expect
from tornado.testing import gen_test

from tests.handlers.test_base_handler import (
    EXIFTOOL_AVAILABLE,
    JPEGTRAN_AVAILABLE,
    BaseImagingTestCase,
)
from thumbor.config import Config
from thumbor.context import Context, ServerParameters
from thumbor.importer import Importer

# pylint: disable=broad-except,abstract-method,attribute-defined-outside-init,line-too-long,too-many-public-methods
# pylint: disable=too-many-lines


@pytest.mark.skipif(
    not JPEGTRAN_AVAILABLE or not EXIFTOOL_AVAILABLE,
    reason="In order to run thumbor's tests please install both jpegtran and exiftool",
)
class ImageOperationsWithJpegtranTestCase(BaseImagingTestCase):
    def get_context(self):
        cfg = Config(SECURITY_KEY="ACME-SEC")
        cfg.LOADER = "thumbor.loaders.file_loader"
        cfg.FILE_LOADER_ROOT_PATH = self.loader_path
        cfg.JPEGTRAN_PATH = which("jpegtran")
        cfg.PROGRESSIVE_JPEG = (True,)
        cfg.RESULT_STORAGE_STORES_UNSAFE = (True,)
        cfg.OPTIMIZERS = [
            "thumbor.optimizers.jpegtran",
        ]

        importer = Importer(cfg)
        importer.import_modules()
        server = ServerParameters(
            8889, "localhost", "thumbor.conf", None, "info", None
        )
        server.security_key = "ACME-SEC"
        ctx = Context(server, cfg, importer)
        return ctx

    @gen_test
    async def test_should_optimize_jpeg(self):
        response = await self.async_fetch("/unsafe/200x200/image.jpg")

        tmp_fd, tmp_file_path = tempfile.mkstemp(suffix=".jpg")
        fixture = os.fdopen(tmp_fd, "wb")
        fixture.write(response.body)
        fixture.close()

        exiftool = which("exiftool")
        if not exiftool:
            raise AssertionError(
                "exiftool was not found. Please install it to run thumbor's tests."
            )

        command = [exiftool, tmp_file_path, "-DeviceModel", "-EncodingProcess"]

        try:
            output = subprocess.check_output(command, stdin=subprocess.DEVNULL)

            expect(response.code).to_equal(200)
            expect(output).to_equal(
                "Encoding Process                : Progressive DCT, Huffman coding\n"
            )
        finally:
            os.remove(tmp_file_path)

    @gen_test
    async def test_with_meta(self):
        response = await self.async_fetch("/unsafe/meta/800x400/image.jpg")
        expect(response.code).to_equal(200)

    @gen_test
    async def test_with_meta_cached(self):
        await self.async_fetch("/unsafe/meta/800x400/image.jpg")
        response = await self.async_fetch("/unsafe/meta/800x400/image.jpg")
        expect(response.code).to_equal(200)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from preggy import expect
from tornado.testing import gen_test

from tests.base import TestCase
from thumbor.config import Config
from thumbor.context import Context
from thumbor.importer import Importer


class HealthcheckHandlerTestCase(TestCase):
    @gen_test
    async def test_can_get_healthcheck(self):
        response = await self.async_get("/healthcheck")
        expect(response.code).to_equal(200)
        expect(response.body).to_equal("WORKING")
        expect(response.headers.get("Cache-Control")).to_equal("no-cache")

    @gen_test
    async def test_can_head_healthcheck(self):
        response = await self.async_fetch("/healthcheck", method="HEAD")
        expect(response.code).to_equal(200)
        expect(response.headers.get("Cache-Control")).to_equal("no-cache")


# Same test, but configured for the root URL
class HealthcheckOnRootTestCase(TestCase):
    def get_context(self):
        cfg = Config()
        cfg.HEALTHCHECK_ROUTE = "/"

        importer = Importer(cfg)
        importer.import_modules()

        return Context(None, cfg, importer)

    @gen_test
    async def test_can_get_healthcheck(self):
        response = await self.async_get("/")
        expect(response.code).to_equal(200)
        expect(response.body).to_equal("WORKING")

    @gen_test
    async def test_can_head_healthcheck(self):
        response = await self.async_fetch("/", method="HEAD")
        expect(response.code).to_equal(200)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

import tempfile
from os.path import abspath, dirname, join

from preggy import expect
from tornado.testing import gen_test

from tests.base import TestCase
from thumbor.config import Config
from thumbor.context import Context
from thumbor.importer import Importer


class BlacklistHandlerTestCase(TestCase):
    def get_context(self):
        file_storage_root_path = (
            tempfile.TemporaryDirectory().name  # pylint: disable=consider-using-with
        )
        cfg = Config()
        cfg.USE_BLACKLIST = True
        cfg.LOADER = "thumbor.loaders.file_loader"
        cfg.FILE_LOADER_ROOT_PATH = abspath(
            join(dirname(__file__), "../fixtures/images/")
        )
        cfg.STORAGE = "thumbor.storages.file_storage"
        cfg.FILE_STORAGE_ROOT_PATH = file_storage_root_path
        importer = Importer(cfg)
        importer.import_modules()
        return Context(None, cfg, importer)

    @gen_test
    async def test_can_get_blacklist(self):
        response = await self.async_fetch("/blacklist")
        expect(response.code).to_equal(200)
        expect(response.body).to_equal("")

    @gen_test
    async def test_can_put_object_to_blacklist(self):
        response = await self.async_fetch(
            "/blacklist?blocked.jpg", method="PUT", body=""
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal("")

    @gen_test
    async def test_can_read_updated_blacklist(self):
        await self.async_fetch("/blacklist?blocked.jpg", method="PUT", body="")
        response = await self.async_fetch("/blacklist")
        expect(response.code).to_equal(200)
        expect(b"blocked.jpg\n" in response.body).to_equal(True)

    @gen_test
    async def test_cant_get_blacklisted_image(self):
        response = await self.async_fetch("/unsafe/image.jpg")
        expect(response.code).to_equal(200)
        await self.async_fetch("/blacklist?image.jpg", method="PUT", body="")
        response = await self.async_fetch("/unsafe/image.jpg")
        expect(response.code).to_equal(400)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from typing import Dict

from thumbor.detectors.local_detector import CascadeLoaderDetector

HAIR_OFFSET = 0.12


class Detector(CascadeLoaderDetector):
    def __init__(self, context, index, detectors):
        super().__init__(context, index, detectors)
        self.load_cascade_file(
            __file__, self.context.config.FACE_DETECTOR_CASCADE_FILE
        )

    def get_origin(self) -> str:
        return "Face Detection"

    def get_detection_offset(
        self, left: int, top: int, width: int, height: int
    ) -> Dict[str, int]:
        top_offset = -1 * height * HAIR_OFFSET

        if top - height * HAIR_OFFSET < 0.0:
            top_offset = 0.0

        return {"top": top_offset}



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

import shutil
from datetime import datetime
from os.path import dirname, exists, join
from tempfile import mkdtemp
from unittest import mock

from preggy import expect
from tornado.testing import gen_test

from tests.base import TestCase
from thumbor.config import Config
from thumbor.context import ServerParameters
from thumbor.storages.file_storage import Storage as FileStorage


class BaseFileStorageTestCase(TestCase):
    def get_config(self):
        tmp = mkdtemp(suffix="thumbor-file-storage")
        return Config(FILE_STORAGE_ROOT_PATH=tmp)

    def get_server(self):
        server = ServerParameters(
            8888, "localhost", "thumbor.conf", None, "info", None
        )
        server.security_key = "ACME-SEC"
        return server

    def get_image_path(self, name):
        return f"./tests/fixtures/images/{name}"

    def get_image_bytes(self, name):
        with open(self.get_image_path(name), "rb") as img:
            return img.read()

    def get_image_url(self, name):
        return f"s.glbimg.com/some/{name}"


class FileStorageTestCase(BaseFileStorageTestCase):
    @gen_test
    async def test_creates_root_path_if_none_found(self):
        storage = FileStorage(self.context)
        storage.ensure_dir(self.config.FILE_STORAGE_ROOT_PATH)
        expect(exists(self.config.FILE_STORAGE_ROOT_PATH)).to_be_true()

    @gen_test
    async def test_can_store_image_should_be_in_catalog(self):
        url = self.get_image_url("image.jpg")
        image_bytes = self.get_image_bytes("image.jpg")
        storage = FileStorage(self.context)
        await storage.put(url, image_bytes)
        got = await storage.get(url)
        expect(got).not_to_be_null()
        expect(got).not_to_be_an_error()

    @gen_test
    async def test_can_store_image_with_spaces(self):
        url = self.get_image_url("image .jpg")
        image_bytes = self.get_image_bytes("image .jpg")
        storage = FileStorage(self.context)
        await storage.put(url, image_bytes)
        got = await storage.get(url)
        expect(got).not_to_be_null()
        expect(got).not_to_be_an_error()
        expect(got).to_equal(image_bytes)

    @gen_test
    async def test_can_store_image_with_spaces_encoded(self):
        url = self.get_image_url("image%20.jpg")
        image_bytes = self.get_image_bytes("image .jpg")
        storage = FileStorage(self.context)
        await storage.put(url, image_bytes)
        got = await storage.get(url)
        expect(got).not_to_be_null()
        expect(got).not_to_be_an_error()
        expect(got).to_equal(image_bytes)

    @gen_test
    async def test_can_store_images_in_same_folder(self):
        iurl = self.get_image_url("image_999.jpg")
        other_iurl = iurl.replace("/some/", "/some_other/")
        root_path = join(
            self.config.FILE_STORAGE_ROOT_PATH, dirname(other_iurl)
        )
        if exists(root_path):
            shutil.rmtree(root_path)

        storage = FileStorage(self.context)
        ibytes = self.get_image_bytes("image.jpg")
        await storage.put(other_iurl.replace("999", "998"), ibytes)
        await storage.put(other_iurl, ibytes)

        got = await storage.get(other_iurl)
        expect(got).not_to_be_null()
        expect(got).not_to_be_an_error()

    @gen_test
    async def test_can_get_image(self):
        iurl = self.get_image_url("image_2.jpg")
        ibytes = self.get_image_bytes("image.jpg")
        storage = FileStorage(self.context)
        await storage.put(iurl, ibytes)
        got = await storage.get(iurl)
        expect(got).not_to_be_null()
        expect(got).not_to_be_an_error()
        expect(got).to_equal(ibytes)

    @gen_test
    async def test_does_not_store_if_config_says_not_to(self):
        iurl = self.get_image_url("image_5.jpg")
        ibytes = self.get_image_bytes("image.jpg")
        storage = FileStorage(self.context)
        await storage.put(iurl, ibytes)
        await storage.put_crypto(iurl)
        got = await storage.get_crypto(iurl)
        expect(got).to_be_null()

    @gen_test
    async def test_detector_can_store_detector_data(self):
        iurl = self.get_image_url("image_7.jpg")
        ibytes = self.get_image_bytes("image.jpg")
        storage = FileStorage(self.context)
        await storage.put(iurl, ibytes)
        await storage.put_detector_data(iurl, "some-data")
        got = await storage.get_detector_data(iurl)
        expect(got).not_to_be_null()
        expect(got).not_to_be_an_error()
        expect(got).to_equal("some-data")

    @gen_test
    async def test_detector_returns_none_if_no_detector_data(self):
        iurl = self.get_image_url("image_10000.jpg")
        storage = FileStorage(self.context)
        got = await storage.get_detector_data(iurl)
        expect(got).to_be_null()


class ExpiredFileStorageTestCase(BaseFileStorageTestCase):
    def get_config(self):
        return Config(
            FILE_STORAGE_ROOT_PATH="/tmp/thumbor/file_storage/",
            STORAGE_EXPIRATION_SECONDS=10,
        )

    @gen_test
    async def test_cannot_get_expired_1_day_old_image(self):
        iurl = self.get_image_url("image_2.jpg")
        ibytes = self.get_image_bytes("image.jpg")
        storage = FileStorage(self.context)
        await storage.put(iurl, ibytes)
        current_timestamp = (
            datetime.utcnow() - datetime(1970, 1, 1)
        ).total_seconds()
        new_mtime = current_timestamp - 60 * 60 * 24
        with mock.patch(
            "thumbor.storages.file_storage.getmtime", return_value=new_mtime
        ):
            got = await storage.get(iurl)
        expect(got).to_be_null()
        expect(got).not_to_be_an_error()


class ExpirationNoneFileStorageTestCase(BaseFileStorageTestCase):
    def get_config(self):
        return Config(
            FILE_STORAGE_ROOT_PATH="/tmp/thumbor/file_storage/",
            STORAGE_EXPIRATION_SECONDS=None,
        )

    @gen_test
    async def test_can_get_if_expire_set_to_none(self):
        iurl = self.get_image_url("image_2.jpg")
        ibytes = self.get_image_bytes("image.jpg")
        storage = FileStorage(self.context)
        await storage.put(iurl, ibytes)
        got = await storage.get(iurl)
        expect(got).not_to_be_null()
        expect(got).not_to_be_an_error()


class CryptoBadConfFileStorageTestCase(BaseFileStorageTestCase):
    def get_config(self):
        return Config(
            FILE_STORAGE_ROOT_PATH="/tmp/thumbor/file_storage/",
            STORES_CRYPTO_KEY_FOR_EACH_IMAGE=True,
        )

    def get_server(self):
        server = super().get_server()
        server.security_key = ""
        return server

    @gen_test
    async def test_should_be_an_error(self):
        iurl = self.get_image_url("image_3.jpg")
        ibytes = self.get_image_bytes("image.jpg")
        storage = FileStorage(self.context)
        await storage.put(iurl, ibytes)

        msg = "STORES_CRYPTO_KEY_FOR_EACH_IMAGE can't be True if no SECURITY_KEY specified"
        with expect.error_to_happen(
            RuntimeError,
            message=msg,
        ):
            await storage.put_crypto(iurl)


class CryptoFileStorageTestCase(BaseFileStorageTestCase):
    def get_config(self):
        return Config(
            FILE_STORAGE_ROOT_PATH="/tmp/thumbor/file_storage/",
            STORES_CRYPTO_KEY_FOR_EACH_IMAGE=True,
        )

    @gen_test
    async def test_getting_crypto_for_a_new_image_returns_none(self):
        iurl = self.get_image_url("image_9999.jpg")
        storage = FileStorage(self.context)
        got = await storage.get_crypto(iurl)
        expect(got).to_be_null()

    @gen_test
    async def test_can_store_crypto(self):
        iurl = self.get_image_url("image_6.jpg")
        ibytes = self.get_image_bytes("image.jpg")
        storage = FileStorage(self.context)
        await storage.put(iurl, ibytes)
        await storage.put_crypto(iurl)
        got = await storage.get_crypto(iurl)
        expect(got).not_to_be_null()
        expect(got).not_to_be_an_error()
        expect(got).to_equal("ACME-SEC")



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

import mimetypes
import uuid

from thumbor.engines import BaseEngine
from thumbor.handlers import ImageApiHandler


##
# Handler to upload images.
# This handler support only POST method, but images can be uploaded  :
#   - through multipart/form-data (designed for forms)
#   - or with the image content in the request body (rest style)
##
class ImageUploadHandler(ImageApiHandler):
    async def post(self):
        # Check if the image uploaded is a multipart/form-data
        if self.multipart_form_data():
            file_data = self.request.files["media"][0]
            body = file_data["body"]

            # Retrieve filename from 'filename' field
            filename = file_data["filename"]
        else:
            body = self.request.body

            # Retrieve filename from 'Slug' header
            filename = self.request.headers.get("Slug")

        # Check if the image uploaded is valid
        if self.validate(body):

            # Use the default filename for the uploaded images
            if not filename:
                content_type = self.request.headers.get(
                    "Content-Type", BaseEngine.get_mimetype(body)
                )
                extension = mimetypes.guess_extension(
                    content_type.split(";", 1)[0], False
                )
                if extension is None:  # Content-Type is unknown, try with body
                    extension = mimetypes.guess_extension(
                        BaseEngine.get_mimetype(body), False
                    )
                if extension == ".jpe":
                    extension = (
                        ".jpg"  # Hack because mimetypes return .jpe by default
                    )
                if (
                    extension is None
                ):  # Even body is unknown, return an empty string to be contat
                    extension = ""
                filename = (
                    self.context.config.UPLOAD_DEFAULT_FILENAME + extension
                )

            # Build image id based on a random uuid (32 characters)
            image_id = str(uuid.uuid4().hex)
            await self.write_file(image_id, body)
            self.set_status(201)
            self.set_header("Location", self.location(image_id, filename))

    def multipart_form_data(self):
        if (
            "media" not in self.request.files
            or not self.request.files["media"]
        ):
            return False
        return True

    def location(self, image_id, filename):
        base_uri = self.request.uri
        return f"{base_uri}/{image_id}/{filename}"



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from os.path import abspath
from unittest import mock

from preggy import expect
from tornado.testing import gen_test

import thumbor.detectors
from tests.base import DetectorTestCase
from thumbor.detectors.feature_detector import Detector as FeatureDetector


class FeatureDetectorTestCase(DetectorTestCase):
    @gen_test
    async def test_should_detect_multiple_points(self):
        with open(
            abspath("./tests/fixtures/images/no_face.jpg"), "rb"
        ) as fixture:
            self.engine.load(fixture.read(), None)

        await FeatureDetector(self.context, 0, None).detect()
        detection_result = self.context.request.focal_points
        expect(len(detection_result)).to_be_greater_than(1)
        expect(detection_result[0].origin).to_equal("alignment")

    @gen_test
    async def test_should_detect_a_single_point(self):
        with open(
            abspath("./tests/fixtures/images/single_point.jpg"), "rb"
        ) as fixture:
            self.engine.load(fixture.read(), None)

        await FeatureDetector(self.context, 0, None).detect()
        detection_result = self.context.request.focal_points
        expect(len(detection_result)).to_equal(1)
        expect(detection_result[0].origin).to_equal("alignment")

    @gen_test
    async def test_should_not_detect_points(self):
        with open(abspath("./tests/fixtures/images/1x1.png"), "rb") as fixture:
            self.engine.load(fixture.read(), None)

        await FeatureDetector(self.context, 0, []).detect()
        detection_result = self.context.request.focal_points
        expect(detection_result).to_length(0)

    @gen_test
    async def test_should_skip_if_opencv_not_found(self):
        with open(
            abspath("./tests/fixtures/images/no_face.jpg"), "rb"
        ) as fixture:
            self.engine.load(fixture.read(), None)

        with mock.patch.object(
            thumbor.detectors.BaseDetector, "verify_cv", lambda self: False
        ):
            await FeatureDetector(self.context, 0, None).detect()

        detection_result = self.context.request.focal_points
        expect(detection_result).to_length(0)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from unittest import TestCase, mock

from preggy import expect

from thumbor.config import Config
from thumbor.context import (
    Context,
    ContextImporter,
    RequestParameters,
    ServerParameters,
    ThreadPool,
)
from thumbor.filters import FiltersFactory
from thumbor.importer import Importer
from thumbor.metrics.logger_metrics import Metrics


class ContextTestCase(TestCase):
    @staticmethod
    def test_can_create_context():
        ctx = Context()

        expect(ctx.server).to_be_null()
        expect(ctx.config).to_be_null()
        expect(ctx.modules).to_be_null()

        expect(ctx.metrics).not_to_be_null()
        expect(ctx.metrics).to_be_instance_of(Metrics)

        expect(ctx.filters_factory).to_be_instance_of(FiltersFactory)
        expect(ctx.filters_factory.filter_classes_map).to_be_empty()
        expect(ctx.request_handler).to_be_null()
        expect(ctx.thread_pool).to_be_instance_of(ThreadPool)
        expect(ctx.headers).to_be_instance_of(dict)
        expect(ctx.headers).to_be_empty()

    @staticmethod
    def test_can_create_context_with_importer():
        cfg = Config()
        importer = Importer(cfg)
        importer.import_modules()
        ctx = Context(config=cfg, importer=importer)

        expect(ctx.modules).not_to_be_null()
        expect(ctx.modules.importer).to_equal(importer)

    @staticmethod
    def test_can_create_context_without_importer_metrics():
        cfg = Config(
            METRICS="",
        )
        importer = Importer(cfg)
        ctx = Context(config=cfg, importer=importer)

        expect(ctx.modules).not_to_be_null()
        expect(ctx.modules.importer).to_equal(importer)

    @staticmethod
    def test_can_config_define_app_class():
        server = ServerParameters(
            port=8888,
            ip="0.0.0.0",
            config_path="/my/config_path.conf",
            keyfile="./tests/fixtures/thumbor.key",
            log_level="debug",
            app_class="thumbor.app.ThumborServiceApp",
        )

        cfg = Config(
            APP_CLASS="config.app",
        )
        importer = Importer(cfg)
        ctx = Context(config=cfg, importer=importer, server=server)

        expect(ctx.app_class).to_equal("config.app")

    @staticmethod
    def test_can_server_app_class_override_config():
        server = ServerParameters(
            port=8888,
            ip="0.0.0.0",
            config_path="/my/config_path.conf",
            keyfile="./tests/fixtures/thumbor.key",
            log_level="debug",
            app_class="server.app",
        )

        cfg = Config(
            APP_CLASS="config.app",
        )
        importer = Importer(cfg)
        ctx = Context(config=cfg, importer=importer, server=server)

        expect(ctx.app_class).to_equal("server.app")


class ServerParametersTestCase(TestCase):
    @staticmethod
    def test_can_create_server_parameters():
        params = ServerParameters(
            port=8888,
            ip="0.0.0.0",
            config_path="/my/config_path.conf",
            keyfile="./tests/fixtures/thumbor.key",
            log_level="debug",
            app_class="app",
            fd="fd",
            gifsicle_path="gifsicle_path",
        )

        expect(params.port).to_equal(8888)
        expect(params.ip).to_equal("0.0.0.0")
        expect(params.config_path).to_equal("/my/config_path.conf")
        expect(params.keyfile).to_equal("./tests/fixtures/thumbor.key")
        expect(params.log_level).to_equal("debug")
        expect(params.app_class).to_equal("app")
        expect(
            params._security_key  # pylint: disable=protected-access
        ).to_equal("SECURITY_KEY_FILE")
        expect(params.fd).to_equal("fd")
        expect(params.gifsicle_path).to_equal("gifsicle_path")

        expect(params.security_key).to_equal("SECURITY_KEY_FILE")

    @staticmethod
    def test_can_set_security_key():
        params = ServerParameters(
            port=8888,
            ip="0.0.0.0",
            config_path="/my/config_path.conf",
            keyfile="./tests/fixtures/thumbor.key",
            log_level="debug",
            app_class="app",
            fd="fd",
            gifsicle_path="gifsicle_path",
        )

        params.security_key = "testé"
        expect(
            params._security_key  # pylint: disable=protected-access
        ).to_equal("testé".encode("utf-8"))

    @staticmethod
    def test_loading_does_nothing_if_no_keyfile():
        params = ServerParameters(
            port=8888,
            ip="0.0.0.0",
            config_path="/my/config_path.conf",
            keyfile=None,
            log_level="debug",
            app_class="app",
            fd="fd",
            gifsicle_path="gifsicle_path",
        )
        expect(
            params._security_key  # pylint: disable=protected-access
        ).to_be_null()

    @staticmethod
    def test_cant_load_invalid_security_key_file():
        expected_msg = (
            "Could not find security key file at /bogus."
            " Please verify the keypath argument."
        )
        with expect.error_to_happen(ValueError, message=expected_msg):
            ServerParameters(
                port=8888,
                ip="0.0.0.0",
                config_path="/my/config_path.conf",
                keyfile="/bogus",
                log_level="debug",
                app_class="app",
                fd="fd",
                gifsicle_path="gifsicle_path",
            )


class RequestParametersTestCase(TestCase):
    @staticmethod
    def test_can_create_request_parameters():
        params = RequestParameters()

        expect(params.debug).to_be_false()
        expect(params.meta).to_be_false()
        expect(params.trim).to_be_null()
        expect(params.crop).to_be_like(
            {"top": 0, "right": 0, "bottom": 0, "left": 0}
        )
        expect(params.should_crop).to_be_false()

        expect(params.adaptive).to_be_false()
        expect(params.full).to_be_false()
        expect(params.fit_in).to_be_false()
        expect(params.stretch).to_be_false()

        expect(params.width).to_equal(0)
        expect(params.height).to_equal(0)

        expect(params.horizontal_flip).to_be_false()
        expect(params.vertical_flip).to_be_false()

        expect(params.halign).to_equal("center")
        expect(params.valign).to_equal("middle")

        expect(params.smart).to_be_false()

        expect(params.filters).to_be_empty()
        expect(params.image_url).to_be_null()
        expect(params.url).to_be_null()
        expect(params.detection_error).to_be_null()
        expect(params.quality).to_equal(80)
        expect(params.buffer).to_be_null()

        expect(params.focal_points).to_be_empty()

        expect(params.hash).to_be_null()
        expect(params.prevent_result_storage).to_be_false()
        expect(params.unsafe).to_be_false()
        expect(params.format).to_be_null()
        expect(params.accepts_webp).to_be_false()
        expect(params.max_bytes).to_be_null()
        expect(params.max_age).to_be_null()

    @staticmethod
    def test_can_get_params_with_trim():
        params = RequestParameters(trim="trim")
        expect(params.trim_pos).to_equal("top-left")
        expect(params.trim_tolerance).to_equal(0)

    @staticmethod
    def test_can_get_params_with_trim_with_custom_pos():
        params = RequestParameters(trim="trim:bottom-right:10")
        expect(params.trim_pos).to_equal("bottom-right")
        expect(params.trim_tolerance).to_equal(10)

    @staticmethod
    def test_can_get_params_with_crop():
        params = RequestParameters(
            crop_left=10,
            crop_right=20,
            crop_top=30,
            crop_bottom=40,
        )
        expect(params.crop).to_be_like(
            {"top": 30, "right": 20, "bottom": 40, "left": 10}
        )

    @staticmethod
    def test_can_get_params_with_custom_crop():
        params = RequestParameters(
            crop={"top": 30, "right": 20, "bottom": 40, "left": 10}
        )
        expect(params.crop).to_be_like(
            {"top": 30, "right": 20, "bottom": 40, "left": 10}
        )
        expect(params.should_crop).to_be_true()

    @staticmethod
    def test_can_get_orig_dimensions():
        params = RequestParameters(
            width="orig",
            height="orig",
        )
        expect(params.width).to_equal("orig")
        expect(params.height).to_equal("orig")

    @staticmethod
    def test_can_add_filters():
        params = RequestParameters(filters=["a", "b"])
        expect(params.filters).to_length(2)

    @staticmethod
    def test_can_add_focal_points():
        params = RequestParameters(focal_points=["a", "b"])
        expect(params.focal_points).to_length(2)

    @staticmethod
    def test_can_get_params_from_request():
        request = mock.Mock(path="/test.jpg", headers={"Accept": "image/webp"})
        params = RequestParameters(request=request, image="/test.jpg")
        expect(params.accepts_webp).to_be_true()
        expect(params.image_url).to_equal("/test.jpg")


class ContextImporterTestCase(TestCase):
    @staticmethod
    def test_can_create_context_importer():
        cfg = Config(
            RESULT_STORAGE="thumbor.result_storages.file_storage",
        )
        importer = Importer(cfg)
        importer.import_modules()
        ctx = Context(config=cfg, importer=importer)

        ctx_importer = ContextImporter(ctx, importer)
        expect(ctx_importer.context).to_equal(ctx)
        expect(ctx_importer.importer).to_equal(importer)
        expect(ctx_importer.engine).to_be_instance_of(importer.engine)
        expect(ctx_importer.gif_engine).to_be_instance_of(importer.gif_engine)

        expect(ctx_importer.storage).to_be_instance_of(importer.storage)
        expect(ctx_importer.result_storage).to_be_instance_of(
            importer.result_storage
        )
        expect(ctx_importer.upload_photo_storage).to_be_instance_of(
            importer.upload_photo_storage
        )

        expect(ctx_importer.loader).to_equal(importer.loader)
        expect(ctx_importer.detectors).to_equal(importer.detectors)
        expect(ctx_importer.filters).to_equal(importer.filters)
        expect(ctx_importer.optimizers).to_equal(importer.optimizers)
        expect(ctx_importer.url_signer).to_equal(importer.url_signer)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

import signal
from unittest import TestCase, mock

from thumbor.signal_handler import setup_signal_handler, signal_handler


class SignalhandlerTestCase(TestCase):
    @mock.patch("thumbor.signal_handler.signal")
    def test_setup_signal_handler_sets_handler(self, signal_mock):
        signal_mock.SIGINT = 2
        signal_mock.SIGTERM = 15
        setup_signal_handler(mock.Mock(), mock.Mock())

        signal_mock.signal.assert_has_calls(
            [
                mock.call(signal_mock.SIGTERM, mock.ANY),
                mock.call(signal_mock.SIGINT, mock.ANY),
            ]
        )

    @mock.patch("tornado.ioloop.IOLoop.instance", create=True)
    def test_signal_handler_calls_add_callback_from_signal(self, ioloop_mock):
        ioloop_instance_mock = mock.Mock()
        ioloop_mock.return_value = ioloop_instance_mock

        signal_handler(mock.Mock(), mock.Mock(), signal.SIGTERM, mock.Mock())

        ioloop_instance_mock.add_callback_from_signal.assert_called_with(
            mock.ANY
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

import json
import tempfile

from preggy import expect

from tests.base import TestCase
from thumbor import __version__
from thumbor.config import Config
from thumbor.context import ServerParameters
from thumbor.error_handlers.file import ErrorHandler


class FakeRequest:
    def __init__(self):
        self.headers = {
            "header1": "value1",
            "Cookie": "cookie1=value; cookie2=value2;",
        }

        self.url = "test/"
        self.method = "GET"
        self.arguments = []
        self.body = "body"
        self.query = "a=1&b=2"
        self.remote_ip = "127.0.0.1"

    def full_url(self):
        return f"http://test/{self.url}"


class FakeHandler:
    def __init__(self):
        self.request = FakeRequest()


class InvalidFileErrorHandlerTestCase(TestCase):
    def test_when_invalid_empty_configuration(self):
        with expect.error_to_happen(RuntimeError):
            ErrorHandler(self.config)

    def test_when_invalid_configuration_of_filename_with_context_should_be_error(
        self,
    ):
        cfg = Config(
            ERROR_FILE_NAME_USE_CONTEXT="server..port",
            ERROR_FILE_LOGGER="toto",
        )
        with expect.error_to_happen(RuntimeError):
            ErrorHandler(cfg)


class BasicFileErrorHandlerTestCase(TestCase):
    def get_config(self):
        self.tmp = (
            tempfile.NamedTemporaryFile(  # pylint: disable=consider-using-with
                prefix="thumborTest."
            )
        )
        return Config(SECURITY_KEY="ACME-SEC", ERROR_FILE_LOGGER=self.tmp.name)

    def get_server(self):
        server = ServerParameters(
            8889, "localhost", "thumbor.conf", None, "info", None
        )
        server.security_key = "ACME-SEC"
        return server

    def test_when_error_occurs_should_have_called_client(self):
        handler = ErrorHandler(self.config)
        http_handler = FakeHandler()
        handler.handle_error(self.context, http_handler, RuntimeError("Test"))
        content = self.tmp.read()
        log = json.loads(content.decode("utf-8"))
        del log["extra"]["timestamp"]
        expect(log).to_be_like(
            {
                "Http": {
                    "url": "http://test/test/",
                    "method": "GET",
                    "data": [],
                    "body": "body",
                    "query_string": "a=1&b=2",
                },
                "interfaces.User": {"ip": "127.0.0.1"},
                "exception": "Test",
                "extra": {
                    "thumbor-version": __version__,
                    "Headers": {
                        "header1": "value1",
                        "Cookie": {"cookie1": "value", "cookie2": "value2"},
                    },
                },
            }
        )


class FileErrorHandlerTestCase(TestCase):
    PORT = 8890

    def get_config(self):
        self.tmp = (
            tempfile.NamedTemporaryFile(  # pylint: disable=consider-using-with
                prefix=f"thumborTest.{self.PORT}."
            )
        )
        return Config(
            SECURITY_KEY="ACME-SEC",
            ERROR_FILE_LOGGER=self.tmp.name.replace(
                f"thumborTest.{self.PORT}.", "thumborTest.%i."
            ),
            ERROR_FILE_NAME_USE_CONTEXT="server.port",
        )

    def get_server(self):
        server = ServerParameters(
            self.PORT, "localhost", "thumbor.conf", None, "info", None
        )
        server.security_key = "ACME-SEC"
        return server

    def test_when_error_occurs_i_use_context_should_have_called_client(self):
        handler = ErrorHandler(self.config)
        http_handler = FakeHandler()
        handler.handle_error(self.context, http_handler, RuntimeError("Test"))
        content = self.tmp.read()

        # check against json version
        log = json.loads(content.decode("utf-8"))
        del log["extra"]["timestamp"]
        expect(log).to_be_like(
            {
                "Http": {
                    "url": "http://test/test/",
                    "method": "GET",
                    "data": [],
                    "body": "body",
                    "query_string": "a=1&b=2",
                },
                "interfaces.User": {"ip": "127.0.0.1"},
                "exception": "Test",
                "extra": {
                    "thumbor-version": __version__,
                    "Headers": {
                        "header1": "value1",
                        "Cookie": {"cookie1": "value", "cookie2": "value2"},
                    },
                },
            }
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from preggy import expect
from tornado.testing import gen_test

from tests.fixtures.watermark_fixtures import (
    POSITIONS,
    RATIOS,
    SOURCE_IMAGE_SIZES,
    WATERMARK_IMAGE_SIZES,
)
from thumbor.config import Config
from thumbor.context import Context
from thumbor.filters import watermark
from thumbor.importer import Importer
from thumbor.testing import FilterTestCase


class WatermarkFilterTestCase(FilterTestCase):
    @gen_test
    async def test_watermark_filter_centered(self):
        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.watermark",
            "watermark(watermark.png,center,center,60)",
        )
        expected = self.get_fixture("watermarkCenter.jpg")
        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.98)

    @gen_test
    async def test_watermark_filter_centered_x(self):
        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.watermark",
            "watermark(watermark.png,center,40,20)",
        )
        expected = self.get_fixture("watermarkCenterX.jpg")
        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.98)

    @gen_test
    async def test_watermark_filter_centered_y(self):
        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.watermark",
            "watermark(watermark.png,80,center,50)",
        )
        expected = self.get_fixture("watermarkCenterY.jpg")
        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.98)

    @gen_test
    async def test_watermark_filter_repeated(self):
        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.watermark",
            "watermark(watermark.png,repeat,repeat,70)",
        )
        expected = self.get_fixture("watermarkRepeat.jpg")
        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.98)

    @gen_test
    async def test_watermark_filter_repeated_x(self):
        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.watermark",
            "watermark(watermark.png,repeat,center,70)",
        )
        expected = self.get_fixture("watermarkRepeatX.jpg")
        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.98)

    @gen_test
    async def test_watermark_filter_repeated_y(self):
        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.watermark",
            "watermark(watermark.png,30,repeat,30)",
        )
        expected = self.get_fixture("watermarkRepeatY.jpg")
        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.98)

    @gen_test
    async def test_watermark_filter_detect_extension_simple(self):
        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.watermark",
            "watermark(watermark,30,-50,60)",
        )
        expected = self.get_fixture("watermarkSimple.jpg")
        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.98)

    @gen_test
    async def test_watermark_filter_simple(self):
        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.watermark",
            "watermark(watermark.png,30,-50,60)",
        )
        expected = self.get_fixture("watermarkSimple.jpg")
        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.98)

    @gen_test
    async def test_watermark_filter_calculated(self):
        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.watermark",
            "watermark(watermark.png,4p,-30p,60)",
        )
        expected = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.watermark",
            "watermark(watermark.png,32,-160,60)",
        )
        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.98)

    @gen_test
    async def test_watermark_filter_calculated_center(self):
        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.watermark",
            "watermark(watermark.png,4p,center,60)",
        )
        expected = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.watermark",
            "watermark(watermark.png,32,center,60)",
        )
        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.98)

    @gen_test
    async def test_watermark_filter_calculated_repeat(self):
        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.watermark",
            "watermark(watermark.png,repeat,30p,60)",
        )
        expected = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.watermark",
            "watermark(watermark.png,repeat,160,60)",
        )
        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.98)

    @gen_test
    async def test_watermark_filter_calculated_position(self):
        watermark.Filter.pre_compile()
        filter_instance = watermark.Filter("http://dummy,0,0,0", self.context)

        for length, pos, expected in POSITIONS:
            test = {
                "length": length,
                "pos": pos,
            }

            expect(
                filter_instance.detect_and_get_ratio_position(pos, length)
            ).to_be_equal_with_additional_info(expected, **test)

    @gen_test
    async def test_watermark_filter_simple_big(self):
        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.watermark",
            "watermark(watermarkBig.png,-10,-100,50)",
        )
        expected = self.get_fixture("watermarkSimpleBig.jpg")
        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.98)

    @gen_test
    async def test_watermark_filter_simple_50p_width(self):
        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.watermark",
            "watermark(watermark.png,30,-50,20,50)",
        )
        expected = self.get_fixture("watermarkResize50pWidth.jpg")
        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.98)

    @gen_test
    async def test_watermark_filter_simple_70p_height(self):
        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.watermark",
            "watermark(watermark.png,30,-50,20,none,70)",
        )
        expected = self.get_fixture("watermarkResize70pHeight.jpg")
        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.98)

    @gen_test
    async def test_watermark_filter_simple_60p_80p(self):
        image = await self.get_filtered(
            "source.jpg",
            "thumbor.filters.watermark",
            "watermark(watermark.png,-30,-200,20,60,80)",
        )
        expected = self.get_fixture("watermarkResize60p80p.jpg")
        ssim = self.get_ssim(image, expected)
        expect(ssim).to_be_greater_than(0.98)

    @gen_test
    async def test_watermark_filter_calculated_resizing(self):
        watermark.Filter.pre_compile()
        filter_instance = watermark.Filter("http://dummy,0,0,0", self.context)

        for source_image_width, source_image_height in SOURCE_IMAGE_SIZES:
            for (
                watermark_source_image_width,
                watermark_source_image_height,
            ) in WATERMARK_IMAGE_SIZES:
                for w_ratio, h_ratio in RATIOS:
                    max_width = (
                        source_image_width * (float(w_ratio) / 100)
                        if w_ratio
                        else float("inf")
                    )
                    max_height = (
                        source_image_height * (float(h_ratio) / 100)
                        if h_ratio
                        else float("inf")
                    )
                    w_ratio = float(w_ratio) / 100.0 if w_ratio else False
                    h_ratio = float(h_ratio) / 100.0 if h_ratio else False

                    ratio = (
                        float(watermark_source_image_width)
                        / watermark_source_image_height
                    )

                    (
                        watermark_image_width,
                        watermark_image_height,
                    ) = filter_instance.calc_watermark_size(
                        (source_image_width, source_image_height),
                        (
                            watermark_source_image_width,
                            watermark_source_image_height,
                        ),
                        w_ratio,
                        h_ratio,
                    )
                    watermark_image = (
                        float(watermark_image_width) / watermark_image_height
                    )

                    test = {
                        "source_image_width": source_image_width,
                        "source_image_height": source_image_height,
                        "watermark_source_image_width": watermark_source_image_width,
                        "watermark_source_image_height": watermark_source_image_height,
                        "watermark_image_width": watermark_image_width,
                        "watermark_image_height": watermark_image_height,
                        "w_ratio": w_ratio,
                        "h_ratio": h_ratio,
                    }

                    test["topic_name"] = "watermark_image_width"
                    expect(watermark_image_width).to_fit_into(
                        max_width, **test
                    )
                    test["topic_name"] = "watermark_image_height"
                    expect(watermark_image_height).to_fit_into(
                        max_height, **test
                    )

                    test["topic_name"] = "fill out"
                    expect(
                        (
                            watermark_image_width == max_width
                            or watermark_image_height == max_height
                        )
                    ).to_be_true_with_additional_info(**test)

                    test["topic_name"] = "image ratio"
                    expect(watermark_image).to_almost_equal(ratio, 2, **test)

    @gen_test
    async def test_watermark_validate_allowed_source(self):
        config = Config(
            ALLOWED_SOURCES=[
                "s.glbimg.com",
            ],
            LOADER="thumbor.loaders.http_loader",
        )
        importer = Importer(config)
        importer.import_modules()

        context = Context(config=config, importer=importer)
        filter_instance = watermark.Filter("", context)

        expect(
            filter_instance.validate("https://s2.glbimg.com/logo.jpg")
        ).to_be_false()
        expect(
            filter_instance.validate("https://s.glbimg.com/logo.jpg")
        ).to_be_true()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from os.path import exists
from subprocess import PIPE, Popen

from thumbor.optimizers import BaseOptimizer
from thumbor.utils import logger


class Optimizer(BaseOptimizer):
    def should_run(self, image_extension, image_buffer):
        if image_extension in [".jpg", ".jpeg"]:
            if self.context.config.JPEGTRAN_PATH is None or not exists(
                self.context.config.JPEGTRAN_PATH
            ):
                logger.warning(
                    "jpegtran optimizer enabled but binary JPEGTRAN_PATH does not exist"
                )
                return False
            return True
        return False

    def run_optimizer(self, image_extension, buffer):
        if not self.should_run(image_extension, buffer):
            return buffer

        if "strip_icc" in self.context.request.filters:
            copy_chunks = "comments"
        else:
            # have to copy everything to preserve icc profile
            copy_chunks = "all"

        command = [
            self.context.config.JPEGTRAN_PATH,
            "-copy",
            copy_chunks,
            "-optimize",
        ]

        if self.context.config.PROGRESSIVE_JPEG:
            command += ["-progressive"]

        if self.context.config.JPEGTRAN_SCANS_FILE:
            if exists(self.context.config.JPEGTRAN_SCANS_FILE):
                command += ["-scans", self.context.config.JPEGTRAN_SCANS_FILE]
            else:
                logger.warning("jpegtran optimizer scans file does not exist")

        jpg_process = Popen(  # pylint: disable=consider-using-with
            command, stdin=PIPE, stdout=PIPE, stderr=PIPE
        )
        output_stdout, output_stderr = jpg_process.communicate(buffer)

        if jpg_process.returncode != 0:
            logger.warning(
                "jpegtran finished with non-zero return code (%d): %s",
                jpg_process.returncode,
                output_stderr,
            )
            return buffer

        return output_stdout



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

# pylint: disable=no-member,line-too-long

import argparse
import sys
from importlib import import_module
from os.path import abspath
from shutil import which
from typing import List

from colorama import Fore, init

from thumbor import __release_date__, __version__
from thumbor.config import Config
from thumbor.ext import BUILTIN_EXTENSIONS

CHECK = "✅"
CROSS = "❎"
WARNING = "⚠️"
ERROR = "⛔"
QUESTION = "❓"


def get_options():
    parser = argparse.ArgumentParser(description="thumbor doctor")

    parser.add_argument(
        "-n",
        "--nocolor",
        action="store_true",
        help="Disables coloring of thumbor doctor",
    )

    parser.add_argument(
        "-c",
        "--config",
        default=None,
        help=(
            "thumbor configuration file. If specified "
            "thumbor-doctor can be fine tuned."
        ),
    )

    options = parser.parse_args()

    return {
        "nocolor": options.nocolor,
        "config": options.config,
    }


def header(msg):
    print(f"{Fore.YELLOW}{msg}")


def subheader(msg):
    print(f"{Fore.MAGENTA}{msg}")
    newline()


def print_error(msg):
    print(f"{Fore.RED}{msg}")


def print_success(msg):
    print(f"{Fore.GREEN}{msg}")


def print_warning(msg):
    print(f"{Fore.YELLOW}{msg}")


def print_info(msg):
    print(f"{Fore.CYAN}{msg}")


def newline():
    print()


def check_extensibility_modules(cfg):
    if cfg is None:
        return []

    newline()
    errors = []

    to_check = [
        (
            lambda cfg: True,
            cfg.STORAGE,
            "Storage for source images could not be imported.",
        ),
        (
            lambda cfg: True,
            cfg.LOADER,
            "Loader for source images could not be imported.",
        ),
        (
            lambda cfg: True,
            cfg.RESULT_STORAGE,
            "ResultStorage could not be imported.",
        ),
        (
            lambda cfg: True,
            cfg.ENGINE,
            "Engine for transforming images could not be imported.",
        ),
        (
            lambda cfg: cfg.UPLOAD_ENABLED,
            cfg.UPLOAD_PHOTO_STORAGE,
            "Uploading to thumbor is enabled and the Upload Storage could not be imported.",
        ),
        (
            lambda cfg: True,
            sorted(cfg.DETECTORS),
            "Detector could not be imported.",
        ),
        (
            lambda cfg: True,
            sorted(cfg.FILTERS),
            "Filter could not be imported.",
        ),
        (
            lambda cfg: True,
            sorted(cfg.OPTIMIZERS),
            "Optimizer could not be imported.",
        ),
        (
            lambda cfg: cfg.USE_CUSTOM_ERROR_HANDLING,
            cfg.ERROR_HANDLER_MODULE,
            "Custom error handling is enabled and the "
            "error handler module could not be imported.",
        ),
        (
            lambda cfg: True,
            sorted(cfg.HANDLER_LISTS),
            "Custom http handler could not be imported.",
        ),
    ]

    if any(c[0](cfg) for c in to_check):
        subheader(
            "Verifying extensibility modules found in your thumbor.conf..."
        )

    for should_check, modules, error_message in to_check:
        if not should_check(cfg):
            continue

        if not isinstance(modules, (list, tuple)):
            modules = [modules]

        for module in modules:
            try:
                import_module(module)
                print_success(f"{CHECK} {module}")
            except ImportError as error:
                print_error(f"{CROSS} {module} - {error_message}")
                errors.append(
                    format_error(
                        module,
                        str(error),
                        error_message,
                    )
                )

    if cfg and (
        has_cv_detector(cfg.DETECTORS) or has_redeye_filter(cfg.FILTERS)
    ):
        try:
            import cv2  # noqa pylint: disable=unused-import,import-outside-toplevel
            import numpy  # noqa pylint: disable=unused-import,import-outside-toplevel
        except ImportError as error:
            print_error(
                f"{CROSS} OpenCV Detectors and Filters - {error_message}"
            )
            errors.append(
                format_error(
                    "Could not import OpenCV, so detectors and filters "
                    "that depend on it will not work",
                    str(error),
                    error_message,
                )
            )

    return errors


def has_redeye_filter(filters: List[str]):
    return any(filter_module.endswith("redeye") for filter_module in filters)


def has_cv_detector(detectors: List[str]):
    cv_detectors = [
        "face_detector",
        "feature_detector",
        "glasses_detector",
        "local_detector",
        "profile_detector",
    ]

    for detector in detectors:
        if any(cv_detector.endswith(detector) for cv_detector in cv_detectors):
            return True

    return False


def check_compiled_extensions():
    newline()
    subheader("Verifying thumbor compiled extensions...")
    errors = []

    for extension in BUILTIN_EXTENSIONS:
        ext_name = extension.replace("thumbor.ext.filters.", "")
        try:
            import_module(extension)
            print_success(f"{CHECK} {ext_name}")
        except ImportError as error:
            print_error(f"{CROSS} {ext_name}")
            errors.append(
                format_error(
                    f"Extension {extension}",
                    str(error),
                    (
                        "Extension could not be compiled. "
                        "This will lead to filter being disabled."
                    ),
                )
            )

    return errors


def format_error(dependency, err, msg):
    formatted_msg = "\n\t".join(msg.split("\n"))
    result = f"""
* {dependency}
    Error Message:
        {err}

    Error Description:
        { formatted_msg }
    """
    return result.strip()


def check_modules():
    newline()
    errors = []

    modules = [
        (
            "pycurl",
            "Thumbor works much better with PyCurl. "
            "For more information visit http://pycurl.io/.",
        ),
        (
            "cairosvg",
            "Thumbor uses CairoSVG for reading SVG files. "
            "For more information check https://cairosvg.org/.",
        ),
    ]

    if modules:
        subheader("Verifying libraries support...")

    for module, error_message in modules:
        try:
            import_module(module)  # NOQA
            print_success(f"{CHECK} {module} is installed correctly.")
        except ImportError as error:
            print_error(f"{CROSS} {module} is not installed.")
            print(error_message)
            newline()
            errors.append(format_error(module, str(error), error_message))

    warn_modules = []

    warnings = []
    for module, error_message in warn_modules:
        try:
            import_module(module)  # NOQA
            print_success(f"{CHECK} {module} is installed correctly.")
        except ImportError as error:
            print_warning(f"{CROSS} {module} is not installed.")
            print(error_message)
            newline()
            warnings.append(format_error(module, str(error), error_message))

    return warnings, errors


def check_extensions(cfg):
    newline()

    errors = []
    programs = []

    if cfg is None or "thumbor.optimizers.jpegtran" in cfg.OPTIMIZERS:
        programs.append(
            (
                "jpegtran",
                (
                    "Thumbor uses jpegtran for optimizing JPEG images. "
                    "For more information visit "
                    "https://linux.die.net/man/1/jpegtran."
                ),
            )
        )

    if cfg is None or "thumbor_plugins.optimizers.gifv" in cfg.OPTIMIZERS:
        programs.append(
            (
                "ffmpeg",
                "Thumbor uses ffmpeg for rendering animated images as GIFV. "
                "For more information visit https://www.ffmpeg.org/.",
            )
        )

    if cfg is None or cfg.USE_GIFSICLE_ENGINE:
        programs.append(
            (
                "gifsicle",
                "Thumbor uses gifsicle for better processing of GIF images. "
                "For more information visit https://www.lcdf.org/gifsicle/.",
            )
        )

    if programs:
        subheader("Verifying extension programs...")

    for program, error_message in programs:
        path = which(program)
        if path is None:
            print_error(f"{CROSS} {program} is not installed.")
            print(error_message)
            newline()
            errors.append(
                format_error(
                    program, f"Could not find {program}.", error_message
                )
            )
        else:
            print_success(f"{CHECK} {program} is installed correctly.")

    return errors


def check_security(cfg):
    subheader("Verifying security...")
    errors = []
    warnings = []

    if cfg is None:
        return errors, warnings

    if cfg.SECURITY_KEY == "MY_SECURE_KEY":
        print_error(f"{CROSS} Using default security key.")

        warnings.append(
            format_error(
                "Security",
                "Using default security key configuration in thumbor.conf.",
                "You should specify a unique security key for thumbor or "
                "use a command line param to specify a security key.\n"
                "For more information visit "
                "https://thumbor.readthedocs.io/en/latest/running.html",
            )
        )

    if cfg.ALLOW_UNSAFE_URL:
        print_error(f"{CROSS} Allowing unsafe URLs.")

        errors.append(
            format_error(
                "Security",
                "Unsafe URLs are enabled.",
                "It is STRONGLY recommended that you turn off "
                "ALLOW_UNSAFE_URLS flag in production environments "
                "as this can lead to DDoS attacks against thumbor.\n"
                "For more information visit "
                "https://thumbor.readthedocs.io/en/latest/security.html",
            )
        )

    return errors, warnings


def load_config(config_path):
    cfg = None
    if config_path is not None:
        path = abspath(config_path)
        cfg = Config.load(path, conf_name="thumbor.conf", lookup_paths=[])
        print(f"Using configuration file found at {config_path}")
    return cfg


def configure_colors(nocolor):
    if nocolor:
        init(strip=True, convert=False)
    else:
        init(autoreset=True)


def print_header(print_version=True):
    if print_version:
        newline()
        header(f"Thumbor v{__version__} (of {__release_date__})")
    newline()
    print(
        "Thumbor doctor will analyze your install and verify "
        "if everything is working as expected."
    )


def check_everything(cfg):
    warnings, errors = check_modules()
    errors += check_compiled_extensions()
    errors += check_extensibility_modules(cfg)
    errors += check_extensions(cfg)

    sec_err, sec_warn = check_security(cfg)
    errors += sec_err
    warnings += sec_warn

    newline()
    return warnings, errors


def print_results(warnings, errors):
    if not warnings and not errors:
        print_success("🎉 Congratulations! No errors found! 🎉")
        return

    print_error("😞 Oh no! We found some things that could improve... 😞")
    newline()

    if warnings:
        print_warning(f"{WARNING}Warnings{WARNING}")
        print("\n\n".join([f"{str(err)}" for err in warnings]))
        newline()

    if errors:
        print_error(f"{ERROR}Errors{ERROR}")
        print("\n\n".join([f"{str(err)}" for err in errors]))
        newline()

    subheader(f"{QUESTION}Need Help{QUESTION}")
    print_info(
        "If you don't know how to fix the above problems, please open an issue with thumbor."
    )
    print_info("Don't forget to copy this log and add it to the description.")
    print("Open an issue at https://github.com/thumbor/thumbor/issues/new")


def run_doctor(options, print_version=True, exit_with_error=True):
    cfg = load_config(options["config"])
    configure_colors(options["nocolor"])

    print_header(print_version)
    warnings, errors = check_everything(cfg)
    print_results(warnings, errors)

    if exit_with_error and errors:
        sys.exit(1)


def main():
    """Verifies the current environment for problems with thumbor"""

    options = get_options()
    run_doctor(options, print_version=True, exit_with_error=True)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

import tornado.ioloop
import tornado.web
from libthumbor.url import Url  # type: ignore

from thumbor.handlers.imaging import ImagingHandler


class ThumborServiceApp(tornado.web.Application):
    def __init__(self, context):
        self.context = context
        self.debug = getattr(self.context.server, "debug", False)
        super().__init__(self.get_handlers(), debug=self.debug)

    def get_handlers(self):
        handlers = []

        for handler_list in self.context.modules.importer.handler_lists:
            get_handlers = getattr(handler_list, "get_handlers", None)

            if get_handlers is None:
                continue
            handlers.extend(get_handlers(self.context))

        # Imaging handler (GET)
        handlers.append(
            (Url.regex(), ImagingHandler, {"context": self.context})
        )

        return handlers



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

import hashlib
import os
from datetime import datetime
from json import dumps, loads
from os.path import dirname, exists, getmtime, splitext
from shutil import move
from uuid import uuid4

from thumbor import storages
from thumbor.utils import logger


class Storage(storages.BaseStorage):
    async def put(self, path, file_bytes):
        file_abspath = self.path_on_filesystem(path)
        temp_abspath = f"{file_abspath}.{str(uuid4()).replace('-', '')}"
        file_dir_abspath = dirname(file_abspath)

        logger.debug("creating tempfile for %s in %s...", path, temp_abspath)

        self.ensure_dir(file_dir_abspath)

        with open(temp_abspath, "wb") as _file:
            _file.write(file_bytes)

        logger.debug("moving tempfile %s to %s...", temp_abspath, file_abspath)
        move(temp_abspath, file_abspath)

        return path

    async def put_crypto(self, path):
        if not self.context.config.STORES_CRYPTO_KEY_FOR_EACH_IMAGE:
            return

        file_abspath = self.path_on_filesystem(path)
        file_dir_abspath = dirname(file_abspath)

        self.ensure_dir(file_dir_abspath)

        if not self.context.server.security_key:
            raise RuntimeError(
                "STORES_CRYPTO_KEY_FOR_EACH_IMAGE can't be "
                "True if no SECURITY_KEY specified"
            )

        crypto_path = f"{splitext(file_abspath)[0]}.txt"
        temp_abspath = f"{crypto_path}.{str(uuid4()).replace('-', '')}"
        with open(temp_abspath, "wb") as _file:
            try:
                security_key = self.context.server.security_key.encode()
            except (UnicodeDecodeError, AttributeError):
                security_key = self.context.server.security_key
            _file.write(security_key)

        move(temp_abspath, crypto_path)
        logger.debug(
            "Stored crypto at %s (security key: %s)",
            crypto_path,
            self.context.server.security_key,
        )

        return file_abspath

    async def put_detector_data(self, path, data):
        file_abspath = self.path_on_filesystem(path)

        path = f"{splitext(file_abspath)[0]}.detectors.txt"
        temp_abspath = f"{path}.{str(uuid4()).replace('-', '')}"

        file_dir_abspath = dirname(file_abspath)
        self.ensure_dir(file_dir_abspath)

        with open(temp_abspath, "w", encoding="utf-8") as _file:
            _file.write(dumps(data))

        move(temp_abspath, path)

        return file_abspath

    async def get(self, path):
        abs_path = self.path_on_filesystem(path)

        resource_available = await self.exists(
            path, path_on_filesystem=abs_path
        )
        if not resource_available:
            return None

        with open(self.path_on_filesystem(path), "rb") as source_file:
            return source_file.read()

    async def get_crypto(self, path):
        file_abspath = self.path_on_filesystem(path)
        crypto_file = f"{splitext(file_abspath)[0]}.txt"

        if not exists(crypto_file):
            return None

        with open(crypto_file, "r", encoding="utf-8") as crypto_f:
            return crypto_f.read()

    async def get_detector_data(self, path):
        file_abspath = self.path_on_filesystem(path)
        path = f"{splitext(file_abspath)[0]}.detectors.txt"

        resource_available = await self.exists(path, path_on_filesystem=path)

        if not resource_available:
            return None

        with open(path, "r", encoding="utf-8") as detector_file:
            return loads(detector_file.read())

    def path_on_filesystem(self, path):
        digest = hashlib.sha1(path.encode("utf-8")).hexdigest()
        root_path = self.context.config.FILE_STORAGE_ROOT_PATH.rstrip("/")
        return f"{root_path}/{digest[:2]}/{digest[2:]}"

    async def exists(
        self, path, path_on_filesystem=None
    ):  # pylint: disable=arguments-differ
        if path_on_filesystem is None:
            path_on_filesystem = self.path_on_filesystem(path)
        return os.path.exists(path_on_filesystem) and not self.__is_expired(
            path_on_filesystem
        )

    async def remove(self, path):
        n_path = self.path_on_filesystem(path)
        return os.remove(n_path)

    def __is_expired(self, path):
        if self.context.config.STORAGE_EXPIRATION_SECONDS is None:
            return False
        timediff = datetime.now() - datetime.fromtimestamp(getmtime(path))
        return (
            timediff.total_seconds()
            > self.context.config.STORAGE_EXPIRATION_SECONDS
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

import logging
import shutil
import tempfile
from os.path import abspath, dirname, join
from shutil import which
from unittest import mock
from urllib.parse import quote

import pytest
import tornado.web
from preggy import expect
from tornado.testing import gen_test

import thumbor.engines.pil
from tests.base import TestCase, skip_unless_avif, skip_unless_heif
from tests.fixtures.images import (
    alabama1,
    default_image,
    invalid_quantization,
    space_image,
)
from thumbor.config import Config
from thumbor.context import Context, ServerParameters
from thumbor.engines.pil import Engine
from thumbor.handlers import BaseHandler
from thumbor.importer import Importer

JPEGTRAN_AVAILABLE = which("jpegtran") is not None
EXIFTOOL_AVAILABLE = which("exiftool") is not None


@pytest.fixture(scope="function")
def unittest_caplog(request, caplog):
    request.function.__self__.caplog = caplog


class ErrorHandler(BaseHandler):
    def get(self):
        self._error(403)


class BaseHandlerTestApp(tornado.web.Application):
    def __init__(self, context):
        self.context = context
        super().__init__([(r"/error", ErrorHandler)])


class BaseImagingTestCase(TestCase):
    def setUp(self):
        self.root_path = tempfile.mkdtemp()
        self.loader_path = abspath(
            join(dirname(__file__), "../fixtures/images/")
        )
        self.base_uri = "/image"
        super().setUp()

    def tearDown(self):
        shutil.rmtree(self.root_path)


class ImagingOperationsTestCase(BaseImagingTestCase):
    caplog = None  # can be added by unittest_caplog fixture

    def get_context(self):
        cfg = Config(SECURITY_KEY="ACME-SEC")
        cfg.LOADER = "thumbor.loaders.file_loader"
        cfg.FILE_LOADER_ROOT_PATH = self.loader_path
        cfg.STORAGE = "thumbor.storages.file_storage"
        cfg.FILE_STORAGE_ROOT_PATH = self.root_path
        cfg.QUALITY = "keep"
        cfg.SVG_DPI = 200

        importer = Importer(cfg)
        importer.import_modules()
        server = ServerParameters(
            8889, "localhost", "thumbor.conf", None, "info", None
        )
        server.security_key = "ACME-SEC"
        return Context(server, cfg, importer)

    @gen_test
    async def test_can_get_image(self):
        response = await self.async_fetch("/unsafe/smart/image.jpg")
        expect(response.code).to_equal(200)
        expect(response.body).to_be_similar_to(default_image())

    @gen_test
    async def test_can_get_image_without_extension(self):
        response = await self.async_fetch("/unsafe/smart/image")
        expect(response.code).to_equal(200)
        expect(response.body).to_be_similar_to(default_image())

    @gen_test
    async def test_get_unknown_image_returns_not_found(self):
        response = await self.async_fetch("/unsafe/smart/imag")
        expect(response.code).to_equal(404)

    @gen_test
    async def test_can_get_unicode_image(self):
        enc_url = quote(
            "15967251_212831_19242645_АгатавЗоопарке.jpg".encode("utf-8")
        )
        response = await self.async_fetch(f"/unsafe/{enc_url}")
        expect(response.code).to_equal(200)
        expect(response.body).to_be_similar_to(default_image())

    @gen_test
    async def test_can_get_signed_regular_image(self):
        response = await self.async_fetch(
            "/_wIUeSaeHw8dricKG2MGhqu5thk=/smart/image.jpg"
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_be_similar_to(default_image())

    @gen_test
    async def test_url_without_unsafe_or_hash_fails(self):
        response = await self.async_fetch("/alabama1_ap620%C3%A9.jpg")
        expect(response.code).to_equal(400)

    @gen_test
    async def test_url_without_image(self):
        response = await self.async_fetch("/unsafe/")
        expect(response.code).to_equal(400)

    @gen_test
    async def test_utf8_encoded_image_name_with_encoded_url(self):
        url = "/lc6e3kkm_2Ww7NWho8HPOe-sqLU=/smart/alabama1_ap620%C3%A9.jpg"
        response = await self.async_fetch(url)
        expect(response.code).to_equal(200)
        expect(response.body).to_be_similar_to(alabama1())

    @gen_test
    async def test_url_with_encoded_hash(self):
        url = "/%D1%80=/alabama1_ap620%C3%A9.jpg"
        response = await self.async_fetch(url)
        expect(response.code).to_equal(400)

    @gen_test
    async def test_image_with_spaces_on_url(self):
        response = await self.async_fetch("/unsafe/image%20space.jpg")
        expect(response.code).to_equal(200)
        expect(response.body).to_be_similar_to(space_image())

    @gen_test
    async def test_can_get_image_with_filter(self):
        response = await self.async_fetch(
            "/5YRxzS2yxZxj9SZ50SoZ11eIdDI=/filters:fill(blue)/image.jpg"
        )
        expect(response.code).to_equal(200)

    @gen_test
    async def test_can_get_image_with_invalid_quantization_table(self):
        response = await self.async_fetch("/unsafe/invalid_quantization.jpg")
        expect(response.code).to_equal(200)
        expect(response.body).to_be_similar_to(invalid_quantization())

    @gen_test
    async def test_getting_invalid_image_returns_bad_request(self):
        response = await self.async_fetch("/unsafe/image_invalid.jpg")
        expect(response.code).to_equal(400)

    @gen_test
    async def test_getting_invalid_watermark_returns_bad_request(self):
        response = await self.async_fetch(
            "/unsafe/filters:watermark(boom.jpg,0,0,0)/image.jpg"
        )
        expect(response.code).to_equal(400)

    @gen_test
    async def test_can_read_monochromatic_jpeg(self):
        response = await self.async_fetch("/unsafe/grayscale.jpg")
        expect(response.code).to_equal(200)
        expect(response.body).to_be_jpeg()

    @gen_test
    async def test_can_read_image_with_small_width_and_no_height(self):
        response = await self.async_fetch("/unsafe/0x0:1681x596/1x/image.jpg")
        expect(response.code).to_equal(200)
        expect(response.body).to_be_jpeg()

    @gen_test
    async def test_can_read_cmyk_jpeg(self):
        response = await self.async_fetch("/unsafe/cmyk.jpg")
        expect(response.code).to_equal(200)
        expect(response.body).to_be_jpeg()

    @gen_test
    async def test_can_read_cmyk_jpeg_as_png(self):
        response = await self.async_fetch(
            "/unsafe/filters:format(png)/cmyk.jpg"
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_be_png()

    @gen_test
    async def test_can_read_image_svg_with_px_units_and_convert_png(self):
        response = await self.async_fetch("/unsafe/Commons-logo.svg")
        expect(response.code).to_equal(200)
        expect(response.body).to_be_png()

        engine = Engine(self.context)
        engine.load(response.body, ".png")
        expect(engine.size).to_equal((1024, 1376))

    @gen_test
    async def test_can_read_image_svg_with_inch_units_and_convert_png(self):
        response = await self.async_fetch("/unsafe/Commons-logo-inches.svg")
        expect(response.code).to_equal(200)
        expect(response.body).to_be_png()

        engine = Engine(self.context)
        engine.load(response.body, ".png")
        expect(engine.size).to_equal((2000, 2600))

    @gen_test
    async def test_can_read_8bit_tiff_as_png(self):
        response = await self.async_fetch("/unsafe/gradient_8bit.tif")
        expect(response.code).to_equal(200)
        expect(response.body).to_be_png()

    @gen_test
    async def test_can_read_16bit_lsb_tiff_as_png(self):
        response = await self.async_fetch(
            "/unsafe/gradient_lsb_16bperchannel.tif"
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_be_png()

    @gen_test
    async def test_can_read_16bit_msb_tiff_as_png(self):
        response = await self.async_fetch(
            "/unsafe/gradient_msb_16bperchannel.tif"
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_be_png()

    @skip_unless_avif
    @gen_test
    async def test_avif_format(self):
        response = await self.async_fetch(
            "/unsafe/filters:format(avif)/image.jpg"
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_be_avif()

    @gen_test
    @pytest.mark.usefixtures("unittest_caplog")
    async def test_avif_unavailable_format_jpg(self):
        self.caplog.set_level(logging.WARNING)
        with mock.patch.object(thumbor.engines.pil, "HAVE_AVIF", False):
            response = await self.async_fetch(
                "/unsafe/filters:format(avif)/image.jpg"
            )
            expect(response.code).to_equal(200)
            expect(response.body).to_be_jpeg()
            assert self.caplog.record_tuples == [
                (
                    "thumbor",
                    logging.WARNING,
                    "[PILEngine] AVIF encoding unavailable, defaulting to .jpg",
                )
            ]

    @gen_test
    @pytest.mark.usefixtures("unittest_caplog")
    async def test_avif_unavailable_format_png(self):
        self.caplog.set_level(logging.WARNING)
        with mock.patch.object(thumbor.engines.pil, "HAVE_AVIF", False):
            response = await self.async_fetch(
                "/unsafe/filters:format(avif)/1x1.png"
            )
            expect(response.code).to_equal(200)
            expect(response.body).to_be_png()
            assert self.caplog.record_tuples == [
                (
                    "thumbor",
                    logging.WARNING,
                    "[PILEngine] AVIF encoding unavailable, defaulting to .png",
                )
            ]

    @skip_unless_avif
    @gen_test
    async def test_avif_quality_setting(self):
        self.context.config.QUALITY = 80
        self.context.config.AVIF_QUALITY = 50
        with mock.patch.object(
            Engine,
            "read",
            autospec=True,
            side_effect=Engine.read,
        ) as mock_read:
            response = await self.async_fetch(
                "/unsafe/filters:format(avif)/image.jpg"
            )
            expect(response.code).to_equal(200)
            expect(response.body).to_be_avif()
            mock_read.assert_called_with(mock.ANY, ".avif", 50)

    @gen_test
    async def test_can_read_heif(self):
        response = await self.async_fetch("/unsafe/image.heic")
        expect(response.code).to_equal(200)
        expect(response.body).to_be_heif()

    @skip_unless_heif
    @gen_test
    async def test_heif_format(self):
        response = await self.async_fetch(
            "/unsafe/filters:format(heic)/image.jpg"
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_be_heif()

    @skip_unless_heif
    @gen_test
    async def test_heif_quality_setting(self):
        self.context.config.QUALITY = 80
        self.context.config.HEIF_QUALITY = 50
        with mock.patch.object(
            Engine,
            "read",
            autospec=True,
            side_effect=Engine.read,
        ) as mock_read:
            response = await self.async_fetch(
                "/unsafe/filters:format(heif)/image.jpg"
            )
            expect(response.code).to_equal(200)
            expect(response.body).to_be_heif()
            mock_read.assert_called_with(mock.ANY, ".heif", 50)

    @gen_test
    @pytest.mark.usefixtures("unittest_caplog")
    async def test_heif_unavailable_format_jpg(self):
        self.caplog.set_level(logging.WARNING)
        with mock.patch.object(thumbor.engines.pil, "HAVE_HEIF", False):
            response = await self.async_fetch(
                "/unsafe/filters:format(heif)/image.jpg"
            )
            expect(response.code).to_equal(200)
            expect(response.body).to_be_jpeg()
            assert self.caplog.record_tuples == [
                (
                    "thumbor",
                    logging.WARNING,
                    "[PILEngine] HEIF encoding unavailable, defaulting to .jpg",
                )
            ]

    @gen_test
    @pytest.mark.usefixtures("unittest_caplog")
    async def test_heif_unavailable_format_png(self):
        self.caplog.set_level(logging.WARNING)
        with mock.patch.object(thumbor.engines.pil, "HAVE_HEIF", False):
            response = await self.async_fetch(
                "/unsafe/filters:format(heif)/1x1.png"
            )
            expect(response.code).to_equal(200)
            expect(response.body).to_be_png()
            assert self.caplog.record_tuples == [
                (
                    "thumbor",
                    logging.WARNING,
                    "[PILEngine] HEIF encoding unavailable, defaulting to .png",
                )
            ]
